unit rmComm;

interface

uses
  windows, Dialogs, Classes, Forms, SysUtils,
  adPort, AwUser, AwWin32, OoMisc,
  rmJBus, rmResStr;

var
  FMyParent: TObject;
  FShowErrorMessages: boolean;
  FLastErrorStr: string;
  FComPort: TapdCustomComPort;
  FTransmittingInProgress: boolean;
  FReceivingInProgress: boolean;
  FStopRequest: boolean;
  FCharDelay: integer;
  FReceivedData: boolean;
  FLastCommandString: string;
  FLastReceivedString: string;
  FCmdTimeOut: integer;
  FTimeOutOccured: boolean;
  FReplyRemainTime: integer;
  FMaxSilenceDuration: integer;
  FMaxSilenceOccured: boolean;
  FLineCountWaited: integer;
  FLineCountReached: boolean;
  FOnSendStart: TNotifyEvent;
  FOnSendEnd: TNotifyEvent;
  FOnWaitReplyStart: TNotifyEvent;
  FOnWaitReply: TNotifyEvent;
  FOnWaitReplyEnd: TNotifyEvent;


function  StrToCtrlStr(const s: string): string;
function  CtrlStrToStr(const s: string): string;
function  IsPortAvailable(ComNum: Cardinal): Boolean;
procedure GetValidPortComList(var ComList: TStringList);
procedure CheckComPort;
procedure CheckComPortAssigned;
procedure CheckComPortOpened;
procedure CheckComPortOutBuffFree(DataLen: integer);
procedure SendAsASCII(cmd: string);
procedure SendAsModbusASCII(ASCIIFrame: TModbusASCIIFrame);
procedure SendAsModbusRTU(RTUFrame: TModbusRTUFrame);
function  ReadAsASCII(TimeOut: integer): string;
function  ReadAsModbusASCII(TimeOut: integer): string;
function  ReadAsModbusRTU(TimeOut: integer): string;

function  ASCII_SendAndRead(cmd: string; timeOut: integer): string;
function  Modbus_SendAndRead(
            mbMode: TModbusMode;
            const TXRTUFrame: TModbusRTUFrame;
            const TXASCIIFrame: TModbusASCIIFrame;
            timeOut: integer): string;
function  ModbusRTU_SendAndRead(
            RTUFrame: TModbusRTUFrame;
            waitedRXByteCount: byte;
            timeOut: integer): string;
function  ModbusASCII_SendAndRead(
            ASCIIFrame: TModbusASCIIFrame;
            timeOut: integer): string;

function  Binary_SendAndRead(cmd: string; bHexa: boolean; timeOut: integer): string;
function  SendAsBinary(cmd: string; bHexa: boolean): boolean;

procedure Fax_Send(addr: string);
procedure EMail_Send(addr: string);


const
  UseDispatcherForAvail: Boolean = True;
  ShowPortsInUse: Boolean = True;


implementation

uses
  rmMisc;


function CtrlStrToStr(const S : string) : string;
var
  i, N : Integer;
  NumStr : string;
begin
  Result := '';
  i := 1;
  try
  while i <= length(S) do
    case S[i] of
    '''' :
      begin
        inc(i);
        while (i <= length(S)) and (S[i] <> '''') do begin
          Result := Result + S[i];
          inc(i);
        end;
        if S[i] <> '''' then
          raise Exception.Create('Mismatched '' in string');
        inc(i);
      end;
    '#' :
      begin
        inc(i);
        NumStr := '';
        while (i <= length(S)) and (S[i] in ['$','0'..'9']) do begin
          NumStr := NumStr + S[i];
          inc(i);
        end;
        if (NumStr = '') then
          raise Exception.Create('Constante numrique attendue aprs #');
        N := StrToInt(NumStr);
        if (N < 0) or (N > 255) then
          raise Exception.Create('Constante numrique hors limite');
        if N = 0 then
          raise Exception.Create('#0 ne peut tre stock');
        Result := Result + chr(N);
      end;
    '^' :
      begin
        inc(i);
        if not (S[i] in ['A'..'Z','a'..'z']) then
          raise Exception.Create('Caractre Alphanumrique attendu aprs ^');
        Result := Result + chr(ord(UpCase(S[i])) - ord('A') + 1);
        inc(i);
      end;
    else
      Result := Result + S[i];
      inc(i);
    end;
  except
  end;
end;

function StrToCtrlStr(const S : string) : string;
var
  i : Integer;
  Ctrl, InQuotes : Boolean;
begin
  Ctrl := False;
  for i := 1 to length(S) do
    if (S[i] = '#') or (S[i] = '^') or (S[i] < ' ') or (S[i] > #127) then begin
      Ctrl := True;
      break;
    end;
  if Ctrl then
    begin
      Result := '';
      InQuotes := False;
      for i := 1 to length(S) do
        case S[i] of
        #1..#31 :
          begin
            if InQuotes then begin
              Result := Result + '''';
              InQuotes := False;
            end;
            Result := Result + '^' + chr(ord(S[i]) + ord('A') - 1);
          end;
        #32..#127 :
          begin
            if not InQuotes then begin
              Result := Result + '''';
              InQuotes := True;
            end;
            Result := Result + S[i];
          end;
        #128..#255 :
          begin
            if InQuotes then begin
              Result := Result + '''';
              InQuotes := False;
            end;
            Result := Result + '#' + IntToStr(ord(S[i]));
          end;
        end;
      if InQuotes then
        Result := Result + '''';
    end
  else
    Result := S;
end;

function IsPortAvailable(ComNum: Cardinal): Boolean;

  function MakeComName(const Dest: PChar; const ComNum: Cardinal): PChar;
  begin
    StrFmt(Dest,'\\.\COM%d',[ComNum]);
    MakeComName := Dest;
  end;

var
  ComName : array[0..12] of Char;
  Res : Integer;
  DeviceLayer: TApdBaseDispatcher;
begin
  DeviceLayer := nil;
  try
    if (ComNum = 0) then
      Result := False
    else begin
      if UseDispatcherForAvail then begin
        DeviceLayer  := TApdWin32Dispatcher.Create(nil);
        Res := DeviceLayer.OpenCom(MakeComName(ComName,ComNum), 64, 64);
        if (Res < 0) then
          if ShowPortsInUse then
            Result := GetLastError = DWORD(Abs(ecAccessDenied))
          else
            Result := False
        else begin
          Result := True;
          DeviceLayer.CloseCom;
        end;
      end else begin
        Res := CreateFile(MakeComName(ComName, ComNum),
                 GENERIC_READ or GENERIC_WRITE,
                 0,
                 nil,
                 OPEN_EXISTING,
                 FILE_ATTRIBUTE_NORMAL or
                 FILE_FLAG_OVERLAPPED,
                 0);
        if Res > 0 then begin
          CloseHandle(Res);
          Result := True;
        end else begin
          if ShowPortsInUse then
            Result := GetLastError = DWORD(Abs(ecAccessDenied))
          else
            Result := False;
        end;
      end;
    end;
  finally
    if UseDispatcherForAvail then
      DeviceLayer.Free;
  end;
end;

procedure GetValidPortComList(var ComList: TStringList);
var
  I : Integer;
  S : string;
begin
  ComList.Clear;
  for I := 1 to MaxComHandles do
    if IsPortAvailable(I) then begin
      S := Format('COM%d', [I]);
      ComList.Add(S);
    end;
end;

procedure CheckComPort;
begin
  // verify FComPort is assigned
  CheckComPortAssigned;

  // verify FComPort is opened
  CheckComPortOpened;
end;

procedure CheckComPortAssigned;
begin
  if (FComPort = nil) then begin
    FLastErrorStr := sErrComPortNotAssigned;
    if FShowErrorMessages then
      ShowMessage(sErrComPortNotAssigned);
    abort;
  end;
end;

procedure CheckComPortOpened;
begin
  if (not FComPort.Open) then begin
    FLastErrorStr := sErrComPortNotOpened;
    if FShowErrorMessages then
      ShowMessage(sErrComPortNotOpened);
    abort;
  end;
end;

procedure CheckComPortOutBuffFree(DataLen: integer);
begin
  try
    if DataLen > FComPort.OutBuffFree then
    begin
      FLastErrorStr := sErrComPortNotEnoughOutBuffFree;
      if FShowErrorMessages then
        ShowMessage(sErrComPortNotEnoughOutBuffFree);
      abort;
    end;
  except
  end;
end;

procedure SendAsASCII(cmd: string);
var
  tCharDelay,           // char delay
  i: integer;
begin
  // verify FComPort free buffer size is enough
  CheckComPortOutBuffFree(length(cmd));

  FStopRequest := false;
  FLastCommandString := trim(cmd);

  FTransmittingInProgress := true;

  // Send data (as a single string)
  try
    if assigned(FOnSendStart) then
      FOnSendStart(FMyParent);

    // if chardelay = 0 then send entire string in one step
    if (FCharDelay = 0) and (FComPort.OutBuffFree >= length(cmd)) then
    begin
      FComPort.Output := cmd;
    end
    else

    // if chardelay <> 0 then send string char by char
    begin
      for i := 1 to length(cmd) do
      begin
        if not FComPort.OutBuffFree >= 1 then
          break;
        FComPort.Output := cmd[i];
        tCharDelay := GetTickCount;
        repeat
          application.processMessages;
        until (GetTickCount - tCharDelay) > FCharDelay;
      end;
    end;

    if assigned(FOnSendEnd) then
      FOnSendEnd(FMyParent);
  except
    FLastErrorStr := sErrComPortSendData;
    if FShowErrorMessages then
      ShowMessage(sErrComPortSendData);
    FTransmittingInProgress := false;
    abort;
  end;

  FTransmittingInProgress := false;
end;

procedure SendAsModbusASCII(ASCIIFrame: TModbusASCIIFrame);
var
  cmd: string;
  i: integer;
begin

  // make string to send, with datas (including LRC) contained in ASCIIFrame
  cmd := '';
  for i := 1 to ASCIIFrame.Len - 3 do   // include LRC with -3 ?
    cmd := cmd + chr(ASCIIFrame.data[i]);
  cmd := sModbusASCIIFrameStart + cmd + sModbusASCIIFrameEnd;

  // verify FComPort free buffer size is enough
  CheckComPortOutBuffFree(length(cmd));

  FTransmittingInProgress := true;

  // Send data (as a single string)
  try
    if assigned(FOnSendStart) then
      FOnSendStart(FMyParent);
    FComPort.PutString(cmd);   // or FComPort.Output(cmd)
    if assigned(FOnSendEnd) then
      FOnSendEnd(FMyParent);
  except
    FLastErrorStr := sErrComPortSendData;
    if FShowErrorMessages then
      ShowMessage(sErrComPortSendData);
    FTransmittingInProgress := false;
    abort;
  end;

  FTransmittingInProgress := false;

end;

procedure SendAsModbusRTU(RTUFrame: TModbusRTUFrame);
begin
  // verify FComPort free buffer size is enough
  CheckComPortOutBuffFree(RTUFrame.Len);

  FTransmittingInProgress := true;

  // send data (as bloc)
  try
    FComPort.PutBlock(RTUFrame.data, RTUFrame.Len);
  except
    FLastErrorStr := sErrComPortSendData;
    if FShowErrorMessages then
      ShowMessage(sErrComPortSendData);
    FTransmittingInProgress := false;
    abort;
  end;

  FTransmittingInProgress := false;
end;

function  ReadAsASCII(TimeOut: integer): string;
var
  t,
  tTimeOut,             // timeOut
  tSilence: integer;    // silence during receiving
  RX: boolean;
  FLineCount: integer;
begin
  result := sEmptyString;
  FReceivedData := false;
  FLastReceivedString := sEmptyString;
  FStopRequest := false;
  FReplyRemainTime := TimeOut;
  FCmdTimeOut := TimeOut;
  RX := false;
  FMaxSilenceOccured := false;
  FTimeOutOccured := false;
  FLineCount := 0;
  FLineCountReached := false;

  FReceivingInProgress := true;

  if assigned(FOnWaitReplyStart) then
    FOnWaitReplyStart(FMyParent);

  tSilence := getTickCount;
  tTimeOut := getTickCount;

  repeat
    application.processMessages;

    if FStopRequest then begin
      FStopRequest := false;
      break;
    end;

    t := GetTickCount;
    FTimeOutOccured := ((t - tTimeOut) > FCmdTimeOut);
    FMaxSilenceOccured := ((t - tSilence) > FMaxSilenceDuration);
    FReplyRemainTime := (FCmdTimeOut - (t - tTimeOut)) div 1000;

    if FReceivedData then begin
      if FLineCountWaited > 0 then begin
        inc(FLineCount);
        FLineCountReached := (FLineCount >= FLineCountWaited);
      end;
      RX := true;
      FReceivedData := false;
      FMaxSilenceOccured := false;
      tSilence := t;
    end;

    if assigned(FOnWaitReply) then
      FOnWaitReply(FMyParent);

    if (not FTimeOutOccured) and (FMaxSilenceOccured) and (not RX) then begin
      tSilence := GetTickCount;
      FMaxSilenceOccured := false;
    end;


  until ((FTimeOutOccured and FMaxSilenceOccured)
     or (RX and FMaxSilenceOccured)
     or FStopRequest
     or FLineCountReached);

  if assigned(FOnWaitReplyEnd) then
    FOnWaitReplyEnd(FMyParent);

  result := FLastReceivedString;
  FReceivingInProgress := false;

end;

function  ReadAsModbusASCII(TimeOut: integer): string;
begin
  FReceivingInProgress := true;
  result := 'ReadAsModbusASCII method not implemented'; // sEmptyString;
  FReceivingInProgress := false;
end;

function  ReadAsModbusRTU(TimeOut: integer): string;
begin
  FReceivingInProgress := true;
  result := 'ReadAsModbusRTU method not implemented'; // sEmptyString;
  FReceivingInProgress := false;
end;

function ASCII_SendAndRead(cmd: string; timeOut: integer): string;
begin
  // send command
  SendAsASCII(cmd);

  // read reply
  result := ReadAsASCII(TimeOut);

end;

function Modbus_SendAndRead(
           mbMode: TModbusMode;
           const TXRTUFrame: TModbusRTUFrame;
           const TXASCIIFrame: TModbusASCIIFrame;
           timeOut: integer): string;
begin
  // RTU
  if mbMode = mmRTU then begin
    result := ModbusRTU_SendAndRead(TXRTUFrame, 0, Timeout);

  // ASCII
  end else if mbMode = mmASCII then begin
    result := ModbusASCII_SendAndRead(TXASCIIFrame, Timeout);

  end;
end;

function ModbusRTU_SendAndRead(
           RTUFrame: TModbusRTUFrame;
           waitedRXByteCount: byte;
           timeOut: integer): string;
begin
  result := '';

  // send data as bloc
  SendAsModbusRTU(RTUFrame);

  // wait answer
  Result := ReadAsModbusRTU(TimeOut);  // not implemented at the moment

end;

function ModbusASCII_SendAndRead(
           ASCIIFrame: TModbusASCIIFrame;
           timeOut: integer): string;
begin
  result := '';

  // Send data
  SendAsModbusASCII(ASCIIFrame);

  // Result := ReadAsModbusASCII;  // to use when implemented
  Result := ReadAsASCII(TimeOut);

end;

function Binary_SendAndRead(cmd: string; bHexa: boolean; timeOut: integer): string;
begin

  // send command
  SendAsBinary(cmd, bHexa);

  // read reply
  result := ReadAsASCII(TimeOut);

end;

function SendAsBinary(cmd: string; bHexa: boolean): boolean;
var
  data: array[0..1000] of byte;
  i, len, sep: integer;
  sTmp: string;
  DataStr: string[3];
  DataByte: byte;
begin
  // verify FComPort free buffer size is enough
  CheckComPortOutBuffFree(length(cmd));

  FStopRequest := false;
  FLastCommandString := trim(cmd);

  FTransmittingInProgress := true;

  result := false;
  if cmd = '' then exit;
  len := 0;
  DataStr := '';
  sTmp := cmd;

  // suppress space in sData begin and end
  sTmp := trim(sTmp);

  // extract bytes from haxa datas and put them into data array
  if bHexa then begin
    while pos(' ', sTmp) > 0 do
      delete(sTmp, pos(' ', sTmp), 1);
    while length(sTmp) > 0 do begin
      DataStr := copy(sTmp, 1, 2);
      dataByte := HexToByte(dataStr);
      data[len] := dataByte;
      inc(len);
      delete(sTmp, 1, 2);
    end;

  // extract bytes from decimal datas and put them into data array
  end else begin
    while length(sTmp) > 0 do begin
      sep := pos(' ', sTmp);
      if sep > 0 then begin
        DataStr := copy(sTmp, 1, sep - 1);
        delete(sTmp, 1, sep);
      end else begin
        DataStr := sTmp;
        sTmp := '';
      end;
      try
        i := StrToInt(dataStr);
        if i in [0..255] then begin
          dataByte := i;
          data[len] := dataByte;
          inc(len);
        end;
      except
      end;
    end;
  end;

  // if no data to send, then exit now
  if len = 0 then exit;

  // send data
  try
    if FComPort.OutBuffFree >= len then
      FComPort.putBlock(data, len);
    result := true;
  except
  end;

end;

procedure Fax_Send(addr: string);
begin
  // not implemented at then moment
end;

procedure EMail_Send(addr: string);
begin
  // not implemented at then moment
end;


end.
