unit rmForms;

interface

uses
  Forms, SysUtils, IniFiles;

function  ShowForm(hFrm: TForm; bShowModal: boolean): integer;
procedure CloseForm(hFrm: TForm);
procedure SetFormSizeAndPos(hFrm: TForm; iLeft, iTop, iWidth, iHeight: integer);
procedure SaveFormSizeAndPos(sFile: string; hFrm: TForm);
procedure SaveFormsSizeAndPos(sFile: string);
procedure LoadFormSizeAndPos(sFile: string; hFrm: TForm);
procedure LoadFormsSizeAndPos(sFile: string);

implementation


function ShowForm(hFrm: TForm; bShowModal: boolean): integer;
begin
  result := -1;
  try
    if hFrm.WindowState = wsMinimized then
      hFrm.WindowState := wsNormal;
    if bShowModal then
      result := hFrm.showModal
    else
      hFrm.show;
  except
  end;
end;

procedure CloseForm(hFrm: TForm);
begin
  try
    hFrm.close;
  except
  end;
end;

procedure SetFormSizeAndPos(hFrm: TForm; iLeft, iTop, iWidth, iHeight: integer);
begin
  try

    if iLeft > -1 then
      hFrm.left := iLeft;
    if iTop > -1 then
      hFrm.Top := iTop;
    if (hFrm.BorderStyle = bsSizeable) or
       (hFrm.BorderStyle = bsSizeToolWin) then
    begin
      if iWidth > -1 then
        hFrm.width := iWidth;
      if iHeight > -1 then
        hFrm.height := iHeight;
    end;

    // Out of screen ?
    if hFrm.left + hFrm.Width > screen.Width then
      hFrm.left := (screen.Width - hFrm.Width) div 2;
    if hFrm.top + hFrm.Height > screen.height then
      hFrm.top := (screen.height - hFrm.height) div 2;

    // maximized or near maximize?
    if (hFrm.Width > (screen.Width - 5)) and
       (hFrm.Height > (screen.Height - 5)) then
    begin
      hFrm.Width := screen.Width - 10;
      hFrm.Height := screen.Height - 10;
      hFrm.top := 0;
      hFrm.Left := 0;
    end;

  except
  end;
end;

procedure SaveFormSizeAndPos(sFile: string; hFrm: TForm);
var
  ini: TIniFile;
  section, key: string;
begin
  section := 'FormsSizeAndPos';
  Ini := TIniFile.create(sFile);
  try
    key := (hFrm as TForm).Name;
    ini.WriteInteger(section, key + '_Left', hFrm.Left);
    ini.WriteInteger(section, key + '_Top', hFrm.top);
    ini.WriteInteger(section, key + '_Width', hFrm.width);
    ini.WriteInteger(section, key + '_Height', hFrm.height);
  finally
    ini.free;
  end;
end;

procedure SaveFormsSizeAndPos(sFile: string);
var
  ini: TIniFile;
  i: integer;
  section, key: string;
begin
  section := 'FormsSizeAndPos';
  Ini := TIniFile.create(sFile);
  try
    for i := 0 to screen.FormCount - 1 do begin
      key := screen.Forms[i].Name;
      ini.WriteInteger(section, key + '_Left', screen.Forms[i].Left);
      ini.WriteInteger(section, key + '_Top', screen.Forms[i].top);
      ini.WriteInteger(section, key + '_Width', screen.Forms[i].width);
      ini.WriteInteger(section, key + '_Height', screen.Forms[i].height);
    end;
  finally
    ini.free;
  end;
end;

procedure LoadFormSizeAndPos(sFile: string; hFrm: TForm);
var
  ini: TIniFile;
  section, key: string;
  iLeft, iTop, iWidth, iHeight: integer;
begin
  if not fileExists(sFile) then exit;

  section := 'FormsSizeAndPos';
  Ini := TIniFile.create(sFile);
  try
    key := (hFrm as TForm).Name;
    iLeft := ini.ReadInteger(section, key + '_Left', -1);
    iTop := ini.ReadInteger(section, key + '_Top', -1);
    iWidth := ini.ReadInteger(section, key + '_Width', -1);
    iHeight := ini.ReadInteger(section, key + '_Height', -1);
    SetFormSizeAndPos(hFrm, iLeft, iTop, iWidth, iHeight);
  finally
    ini.free;
  end;
end;

procedure LoadFormsSizeAndPos(sFile: string);
var
  ini: TIniFile;
  i: integer;
  section, key: string;
  iLeft, iTop, iWidth, iHeight: integer;
begin
  if not fileExists(sFile) then exit;

  section := 'FormsSizeAndPos';
  Ini := TIniFile.create(sFile);
  try
    for i := 0 to screen.FormCount - 1 do begin
      key := screen.Forms[i].Name;
      iLeft := ini.ReadInteger(section, key + '_Left', -1);
      iTop := ini.ReadInteger(section, key + '_Top', -1);
      iWidth := ini.ReadInteger(section, key + '_Width', -1);
      iHeight := ini.ReadInteger(section, key + '_Height', -1);
      SetFormSizeAndPos(screen.forms[i], iLeft, iTop, iWidth, iHeight);
    end;
  finally
    ini.free;
  end;
end;


end.









