unit rmJBus;

interface

uses
  dialogs,
  SysUtils,
  rmMisc;
  //adPort;

type
  TModbusMode = (mmASCII, mmRTU);

  TModbusRTUFrame = record
    Data: array[0..1300] of byte;
    Len: integer;
  end;
  TModbusASCIIFrame = record
    Data: array[0..1300] of byte;
    Len: integer;
  end;

const
  // ModBus Functions (mbf)
  mbfReadNBitOut    = 1;
  mbfReadNBitIn     = 2;
  mbfReadNWordOut   = 3;
  mbfReadNWordIn    = 4;
  mbfWriteNBit      = 5;
  mbfWriteNWord     = 6;
  mbfReadSpeedByte  = 7;
  mbfExchangeDiag   = 8;
  mbfWriteNBitCont  = 15;
  mbfWriteNWordCont = 16;

// common functions
function  MakeModbusFrame(
            var ModbusRTUFrame: TModbusRTUFrame;
            var ModbusASCIIFrame: TModbusASCIIFrame;
            mbMode: TModbusMode;
            SlaveId, FuncId: Byte;
            DataAddr, DataLen: word;
            BitValue: byte;
            BadCRCLRC: boolean): boolean;
function  ModbusFrameToStr(
            var ModbusRTUFrame: TModbusRTUFrame;
            var ModbusASCIIFrame: TModbusASCIIFrame;
            mbMode: TModbusMode): string;
function  ModbusRXStrIsValide(
            sData: string;
            mbMode: TModbusMode): boolean;
function  ModbusCheckData(
            mbMode: TModbusMode;
            RXData, RefStr: string;
            From, Len: integer): boolean;
function  ModbusCommandAccepted(
            mbMode: TModbusMode;
            TXData, RXData: string): boolean;
function  ModbusRXStrExtractBytes(
            mbMode: TModbusMode;
            sData: string;
            var ArrayOfBytes: array of byte): boolean;

// RTU modbus functions
function  AddCRC16(var RTUFrame: TModbusRTUFrame; BadCRC: boolean): boolean;
function  CRC16IsValide(var RTUFrame: TModbusRTUFrame): boolean;
function  MakeModbusRTUFrame(
            var RTUFrame: TModbusRTUFrame;
            SlaveId, FuncId: Byte;
            DataAddr, DataLen: word;
            BitValue: byte;
            BadCRC: boolean): boolean;
function  StrToModbusRTUFrame(
            sData: string;
            var RTUFrame: TModbusRTUFrame;
            BadCRC: boolean): boolean;
function  ModbusRTUFrameIsValide(RTUFrame: TModbusRTUFrame): boolean;
function  ModbusRTUFrameToStr(RTUFrame: TModbusRTUFrame): string;
function  ModbusRTURXStrIsValide(sData: string): boolean;
function  ModbusRTURXStrExtractBytes(
            sData: string;
            var ArrayOfBytes: array of byte): boolean;

// ASCII modbus functions
function  AddLRC16(var ASCIIFrame: TModbusASCIIFrame; BadLRC: boolean): boolean;
function  LRC16IsValide(sData: string): boolean;
function  MakeModbusASCIIFrame(
            var ASCIIFrame: TModbusASCIIFrame;
            SlaveId, FuncId: Byte;
            DataAddr, DataLen: word;
            BitValue: byte;
            BadLRC: boolean): boolean;
function  StrToModbusASCIIFrame(
            sData: string;
            var ASCIIFrame: TModbusASCIIFrame;
            BadLRC: boolean): boolean;
function  ModbusASCIIRXStrIsValide(s: string): boolean;
function  ModbusASCIIFrameToStr(ASCIIFrame: TModbusASCIIFrame): string;
function  ModbusASCIICheckData(
            RXData, RefStr: string): boolean;
function  ModbusASCIICommandAccepted(
            TXData, RXData: string): boolean;
function  ModbusASCIIRXStrExtractBytes(
            sData: string;
            var ArrayOfBytes: array of byte): boolean;


implementation


function  MakeModbusFrame(
            var ModbusRTUFrame: TModbusRTUFrame;
            var ModbusASCIIFrame: TModbusASCIIFrame;
            mbMode: TModbusMode;
            SlaveId, FuncId: Byte;
            DataAddr, DataLen: word;
            BitValue: byte;
            BadCRCLRC: boolean): boolean;
begin
  // RTU
  if mbMode = mmRTU then begin
    MakeModbusRTUFrame(ModbusRTUFrame,
      SlaveId, FuncId,
      DataAddr,
      DataLen,
      BitValue,
      BadCRCLRC);

  // ASCII
  end else if mbMode = mmASCII then begin
    MakeModbusASCIIFrame(ModbusASCIIFrame,
      SlaveId, FuncId,
      DataAddr,
      DataLen,
      BitValue,
      BadCRCLRC);
  end;
end;

function  ModbusFrameToStr(
            var ModbusRTUFrame: TModbusRTUFrame;
            var ModbusASCIIFrame: TModbusASCIIFrame;
            mbMode: TModbusMode): string;
begin
  // RTU
  if mbMode = mmRTU then begin
    result := ModbusRTUFrameToStr(ModbusRTUFrame)

  // ASCII
  end else if mbMode = mmASCII then begin
    result := ModbusASCIIFrameToStr(ModbusASCIIFrame);

  end;
end;

function  ModbusRXStrIsValide(sData: string; mbMode: TModbusMode): boolean;
var
  tmpRTUFrame: TModbusRTUFrame;
begin
  // RTU
  if mbMode = mmRTU then begin
    result := ModbusRTURXStrIsValide(sData);

  // ASCII
  end else if mbMode = mmASCII then begin
    result := ModbusASCIIRXStrIsValide(sData);

  end;
end;

function  ModbusCheckData(
            mbMode: TModbusMode;
            RXData, RefStr: string;
            From, Len: integer): boolean;
begin
  // RTU
  if mbMode = mmRTU then begin
    result := false;

  // ASCII
  end else if mbMode = mmASCII then begin
    result := ModbusASCIICheckData(RXData, RefStr);
  end;
end;

function  ModbusCommandAccepted(
            mbMode: TModbusMode;
            TXData, RXData: string): boolean;
begin
  // RTU
  if mbMode = mmRTU then begin
    result := false;

  // ASCII
  end else if mbMode = mmASCII then begin
    result := ModbusASCIICommandAccepted(TXData, RXData);
  end;
end;

function  ModbusRXStrExtractBytes(
            mbMode: TModbusMode;
            sData: string;
            var ArrayOfBytes: array of byte): boolean;
begin
  // RTU
  if mbMode = mmRTU then begin
    result := ModbusRTURXStrExtractBytes(sData, ArrayOfBytes);

  // ASCII
  end else if mbMode = mmASCII then begin
    result := ModbusASCIIRXStrExtractBytes(sData, ArrayOfBytes);
  end;
end;


function  ModbusASCIIRXStrExtractBytes(
            sData: string;
            var ArrayOfBytes: array of byte): boolean;
var
  BytesToRead: byte;
  val: byte;
  i: integer;
  Err: boolean;
  sTmp: string;
begin
  result := false;
  sTmp := sData;

  // clear data string if necessary
  while pos(' ', sTmp) > 0 do
    delete(sTmp, pos(' ', sTmp), 1);
  while sTmp[1] = ':' do
    delete(sTmp, 1, 1);
  while (sTmp[length(sTmp)] = #13) or (sTmp[length(sTmp)] = #10) do
    delete(sTmp, length(sTmp), 1);

  // supress header, SlaveAddr and Function code
  delete(sTmp, 1, 4);

  // extract bytes to read count
  try
    BytesToRead := HexToByte(copy(sTmp, 1, 2));
  except
    BytesToRead := 0;
  end;
  if BytesToRead = 0 then exit;
  if BytesToRead > (High(ArrayOfBytes) + 1) then
    BytesToRead := (High(ArrayOfBytes) + 1);

  // supress Bytes to read info
  delete(sTmp, 1, 2);

  // read bytes
  Err := false;
  for i := 0 to BytesToRead - 1 do begin
    try
      val := HexToByte(copy(sTmp, 1 + (i * 2), 2));
      ArrayOfBytes[i] := val;
    except
      Err := True;
      ArrayOfBytes[i] := $00;
    end;
  end;

  result := not Err;
end;


function  ModbusASCIICheckData(
            RXData, RefStr: string): boolean;
var
  s1, s2: string;
begin
  // compare RXData (received data) to RefStr (waited Str)
  // only compare the [length(RefStr)] first characters
  s1 := upperCase(RefStr);
  s2 := upperCase(copy(RXData, 1, length(RefStr)));
  result := CompareText(s1, s2) = 0;
end;

function  ModbusASCIICommandAccepted(
            TXData, RXData: string): boolean;
begin
  result := copy(TXData, 1, 5) = copy(RXData, 1, 5);
end;

function  AddCRC16(var RTUFrame: TModbusRTUFrame; BadCRC: boolean): boolean;
var
  i, BitCount, ByteCount, Ovf: integer;
  CRC16, POLY: word;
  wByte: word;
begin
  POLY := 40961 ;     // $A001 - 1010 0000 0000 0001
  CRC16 := 65535 ;    // $FFFF - 1111 1111 1111 1111
  ByteCount := RTUFrame.len;

  for i:= 0 to ByteCount - 1 do begin
    wByte := RTUFrame.data[i];
    CRC16 := CRC16 XOR wByte;
    BitCount := 0;
    while BitCount < 8 do begin
      Ovf := CRC16 AND 1;
      CRC16 := CRC16 SHR 1;
      if (Ovf = 1) then
        CRC16 := CRC16 XOR POLY;
      inc(BitCount);
    end;
  end;

  // add CRC to RTUFrame
  if BadCRC then begin  // volontary add bad CRC (for test when needed)
    RTUFrame.data[ByteCount] := byte(CRC16 AND 255) + 1;    // pf in first
    RTUFrame.data[ByteCount + 1] := byte(CRC16 SHR 8) + 1;  // PF
  end else begin
    RTUFrame.data[ByteCount] := byte(CRC16 AND 255);    // pf in first
    RTUFrame.data[ByteCount + 1] := byte(CRC16 SHR 8);  // PF
  end;
  RTUFrame.len := ByteCount + 2;

  Result := True;
end;

function  CRC16IsValide(var RTUFrame: TModbusRTUFrame): boolean;
begin
  result := false;
end;

function MakeModbusRTUFrame(
           var RTUFrame: TModbusRTUFrame;
           SlaveId, FuncId: Byte;
           DataAddr, DataLen: word;
           BitValue: byte;
           BadCRC: boolean): boolean;
var
  bByte0, bByte1: byte;
begin
  result := false;

  try
    // frame init
    RTUFrame.data[0] := 0;
    RTUFrame.len := 0;

    // frame header
    RTUFrame.data[0] := SlaveId;
    RTUFrame.data[1] := FuncId;
    RTUFrame.data[2] := byte(DataAddr SHR 8);
    RTUFrame.data[3] := byte(DataAddr AND 255);

    // read or write function ?
    // word to read splitted in 2 bytes
    if (FuncId in [1..4, 6]) then begin
      RTUFrame.data[4] := byte((DataLen SHR 8) AND 255);
      RTUFrame.data[5] := byte(DataLen AND 255);
    end else if (FuncId = 5) then begin
      RTUFrame.data[4] := BitValue; // $00 to set bit to 0, or $FF to set bit to 1
      RTUFrame.data[5] := 0;        // always $00
    end;
    RTUFrame.len := 6 ;

    // calc and add CRC
    AddCRC16(RTUFrame, BadCRC);
    result := true;
  except
  end;
end;

function ModbusRTUFrameIsValide(RTUFrame: TModbusRTUFrame): boolean;
begin
  result := CRC16IsValide(RTUFrame);
end;

function ModbusRTUFrameToStr(RTUFrame: TModbusRTUFrame): string;
var
  i: integer;
  s: string;
begin
  s := '';
  if RTUFrame.Len = 0 then exit;
  s := intToHex(RTUFrame.data[0], 2);
  for i := 1 to RTUFrame.Len - 1 do
    s := s + ' ' + intToHex(RTUFrame.data[i], 2);
  result := s;
end;

function  StrToModbusRTUFrame(sData: string; var RTUFrame: TModbusRTUFrame; BadCRC: boolean): boolean;
var
  sTmp: string;
  TmpChar: string[1];
  bSlave, bFunc: byte;
  wAddr, wLen: word;
begin
  result := false;
  RTUFrame.Len := 0;
  if sData = '' then exit;

  sTmp := sData;

  // clear data string if necessary
  while pos(' ', sTmp) > 0 do
    delete(sTmp, pos(' ', sTmp), 1);
  while sTmp[1] = ':' do
    delete(sTmp, 1, 1);
  while (sTmp[length(sTmp)] = #13) or (sTmp[length(sTmp)] = #10) do
    delete(sTmp, length(sTmp), 1);

  // verifying data string length, must be at least 12 char
  if length(sTmp) < 12 then exit;

  // trunc length for actually keep the 12 first char
  sTmp := copy(sTmp, 1, 12);

  // make ASCIIFrame from data string
  bSlave := HexToByte(copy(sTmp, 1, 2));
  bFunc := HexToByte(copy(sTmp, 3, 2));
  wAddr := HexToWord(copy(sTmp, 5, 4));
  wLen := HexToWord(copy(sTmp, 9, 4));

  MakeModbusRTUFrame(RTUFrame, bSlave, bFunc, wAddr, wLen, 0, false);

  result := true;
end;

function ModbusRTURXStrIsValide(sData: string): boolean;
var
  tmpRTUFrame: TModbusRTUFrame;
begin
  result := false;

  strToModbusRTUFrame(sData, tmpRTUFrame, false);
  if not ModbusRTUFrameIsValide(tmpRTUFrame) then
    exit;

  // verifying CRC
  result := CRC16IsValide(tmpRTUFrame);
end;

function  ModbusRTURXStrExtractBytes(
            sData: string;
            var ArrayOfBytes: array of byte): boolean;
var
  ByteToRead: byte;
  val: byte;
  i: integer;
begin
  result := false;

  // extract bytes to read count
  try
    ByteToRead := HexToByte(copy(sData, 6, 2));
  except
    ByteToRead := 0;
  end;
  if ByteToRead = 0 then exit;
  if ByteToRead > (High(ArrayOfBytes) + 1) then
    ByteToRead := (High(ArrayOfBytes) + 1);


  exit;


  // read bytes
  for i := 0 to ByteToRead - 1 do begin
    try
      val := HexToByte(copy(sData, 8 + (i * 2), 2));
      ArrayOfBytes[i] := val;
    except
      ArrayOfBytes[i] := $00;
    end;
  end;

  result := true;
end;

function  AddLRC16(var ASCIIFrame: TModbusASCIIFrame; BadLRC: boolean): boolean;
var
  i, ByteCount: integer;
  bByte, tmpByte: byte;
  LRCLo, LRCHi: string[1];
  c1, c2: char;
  toggle: boolean;
begin
  bByte := 0;
  ByteCount := ASCIIFrame.Len;

  // add all char, first char (colon) excluded
  toggle := false;
  for i := 1 to ByteCount - 1 do begin
    toggle := not toggle;
    if not toggle then continue;
    c1 := chr(ASCIIFrame.Data[i]);
    c2 := chr(ASCIIFrame.Data[i + 1]);
    tmpByte := HexToByte(c1 + c2);
    bByte := bByte + tmpByte;
  end;

  // produce ones-complement, by substract result from FF
  bByte := $FF - bByte;

  // produce two's-complement, by adding 1
  bByte := bByte + 1;

  LRCHi := copy(intToHex(bByte, 2), 1, 1);
  LRCLo := copy(intToHex(bByte, 2), 2, 1);

  // add LRC to ASCIIFrame
  if BadLRC then begin   // volontary add bad LRC (for test when needed)
    ASCIIFrame.Data[ByteCount] := ord(LRCHi[1]) + 1;
    ASCIIFrame.Data[ByteCount + 1] := ord(LRCLo[1]) + 1;
  end else begin
    ASCIIFrame.Data[ByteCount] := ord(LRCHi[1]);
    ASCIIFrame.Data[ByteCount + 1] := ord(LRCLo[1]);
  end;
  ASCIIFrame.Len := ByteCount + 2;

  Result := True;
end;

function  LRC16IsValide(sData: string): boolean;
var
  i: integer;
  TmpByte: byte;
  c1, c2: char;
  sum: byte;
  sTmp: string;
  toggle: boolean;
begin
  // sum of all char (except header colon and ending pair CRLF)
  // must be equal to zero

  result := false;

  if length(sData) < 12 then exit;

  sTmp := sData;

  // clear data string if necessary
  while sTmp[1] = ':' do
    delete(sTmp, 1, 1);
  while (sTmp[length(sTmp)] = #13) or (sTmp[length(sTmp)] = #10) do
    delete(sTmp, length(sTmp), 1);

  // sum of all pair of char (except header colon and ending pair CRLF,
  // previously deleted)
  sum := 0;
  toggle := false;
  for i := 1 to length(sTmp) do begin
    toggle := not toggle;
    if not toggle then continue;
    c1 := sTmp[i];
    c2 := sTmp[i + 1];
    tmpByte := HexToByte(c1 + c2);
    sum := sum + tmpByte;
  end;

  result := (sum = 0);
end;

function MakeModbusASCIIFrame(
           var ASCIIFrame: TModbusASCIIFrame;
           SlaveId, FuncId: Byte;
           DataAddr, DataLen: word;
           BitValue: byte;
           BadLRC: boolean): boolean;
var
  TmpChar: string[1];
  i: integer;
begin
  result := false;

  try
    // start delimitor
    ASCIIFrame.data[0] := ord(':');
    ASCIIFrame.Len := 1;

    // slave addr
    for i := 1 to 2 do begin
      TmpChar := copy(intToHex(SlaveId, 2), i, 1);
      ASCIIFrame.data[0 + i]  := ord(TmpChar[1]);
    end;
    ASCIIFrame.Len := ASCIIFrame.Len + 2;

    // function code
    for i := 1 to 2 do begin
      TmpChar := copy(intToHex(FuncId, 2), i, 1);
      ASCIIFrame.data[2 + i]  := ord(TmpChar[1]);
    end;
    ASCIIFrame.Len := ASCIIFrame.Len + 2;

    // first data word addr to read
    for i := 1 to 4 do begin
      TmpChar := copy(intToHex(DataAddr, 4), i, 1);
      ASCIIFrame.data[4 + i]  := ord(TmpChar[1]);
    end;
    ASCIIFrame.Len := ASCIIFrame.Len + 4;

    // length of data to read (word count)
    for i := 1 to 4 do begin
      TmpChar := copy(intToHex(DataLen, 4), i, 1);
      ASCIIFrame.data[8 + i]  := ord(TmpChar[1]);
    end;
    ASCIIFrame.Len := ASCIIFrame.Len + 4;

    // add LRC char, ASCIIFrame.Len (+ 2 chars) refreshed in LRC calc
    AddLRC16(ASCIIFrame, BadLRC);

    // end delimitor
    ASCIIFrame.data[15] := 13;
    ASCIIFrame.data[16] := 10;
    ASCIIFrame.Len := ASCIIFrame.Len + 2;

    result := true;
  except
  end;
end;

function ModbusASCIIFrameToStr(ASCIIFrame: TModbusASCIIFrame): string;
var
  i: integer;
  s: string;
begin
  result := '';
  s := '';

  if ASCIIFrame.len = 0 then exit;

  // datas, excluding CRLF Pair
  for i := 1 to ASCIIFrame.len - 3 do
    s := s + chr(ASCIIFrame.data[i]);

  result := ':' + s;   //   + ' CR LF';
end;

function  StrToModbusASCIIFrame(
            sData: string;
            var ASCIIFrame: TModbusASCIIFrame;
            BadLRC: boolean): boolean;
var
  sTmp: string;
  TmpChar: string[1];
  bSlave, bFunc: byte;
  wAddr, wLen: word;
begin
  result := false;
  ASCIIFrame.Len := 0;
  if sData = '' then exit;

  sTmp := sData;

  // clear data string if necessary
  while pos(' ', sTmp) > 0 do
    delete(sTmp, pos(' ', sTmp), 1);
  while sTmp[1] = ':' do
    delete(sTmp, 1, 1);
  while (sTmp[length(sTmp)] = #13) or (sTmp[length(sTmp)] = #10) do
    delete(sTmp, length(sTmp), 1);

  // make ASCIIFrame from data string
  bSlave := HexToByte(copy(sTmp, 1, 2));
  bFunc := HexToByte(copy(sTmp, 3, 2));
  wAddr := HexToWord(copy(sTmp, 5, 4));
  wLen := HexToWord(copy(sTmp, 9, 4));

  // make modbus frame
  MakeModbusASCIIFrame(ASCIIFrame, bSlave, bFunc, wAddr, wLen, 0, false);

  result := true;
end;

function ModbusASCIIRXStrIsValide(s: string): boolean;
begin
  result := false;

  if s = '' then exit;
  if length(s) < 3 then exit;

  // verifying header colon, and ending pair CRLF (all assumed detected)
  if (s[1] <> ':') or (s[length(s) - 1] <> #13) or (s[length(s)] <> #10) then
      exit;

  // verifying LRC
  result := LRC16IsValide(s);
end;



end.
