unit rmMaths;

interface

uses
  sysUtils, Dialogs, Math;


function IsBitSet(const i, Nth: integer): boolean;
function BitToOn(const i, Nth: integer): integer;
function BitToOff(const i, Nth: integer): integer;
function BitSwitch(const i, Nth: integer): integer;
function ReverseAllBits(const i: integer): integer;

function Root(base, exponent: extended): extended;

function Log(X: Extended): Extended;
function FactorialX (A: Cardinal): Extended;
{
function Log10(X: Extended): Extended;
function Log2(X: Extended): Extended;
function LogN(Base, X: Extended): Extended;
}
function DegToRad(X: Extended): Extended;

function VppToDBu(r: extended): extended;
function VppToVrms(r: extended): extended;
function VppToDBm(r: extended): extended;
function VppToDev(r: extended): extended;
function VrmsToDbu(r: extended): extended;
function VrmsToVpp(r: extended): extended;
function VrmsToDBm(r: extended): extended;
function VrmsToDev(r: extended): extended;
function DBuToVpp(r: extended): extended;
function DBuToVrms(r: extended): extended;
function DBuToDev(r: extended): extended;
function DevToVpp(r: extended): extended;
function DevToVrms(r: extended): extended;
function DevToDBu(r: extended): extended;

function VppToPower(r: extended; Imp: integer): extended;
function VrmsToPower(r: extended; Imp: integer): extended;
function PowerToVpp(r: extended; Imp: integer): extended;
function PowerToVrms(r: extended; Imp: integer): extended;
function PowerToDBm(r: extended; Imp: integer): extended;
function DBmToPower(ref, r: extended): extended;
function DBmToVpp(r: extended): extended;
function DBmToVrms(r: extended): extended;

function PreEmphasize(Ve, freq, RC: extended; Preemp: boolean): extended;
function DeEmphasize(Ve, freq, RC: extended; Preemp: boolean): extended;

// Old GetMPXMainLevel function version, not take care about time of measure
// Only take care aboute phase between left and right signals
function GetMPXMainLevel(LeftLevel, RightLevel, Phase: extended): extended;

// Old GetMPXSubLevel function version, not take care about time of measure
// Only take care aboute phase between left and right signals
function GetMPXSubLevel(LeftLevel, RightLevel, Phase: extended): extended;

function GetAudioMainLevel(Freq, LeftLevel, RightLevel, LeftPhase, RightPhase, Pilote19KLevel, Pilote19KPhase, Pilote38KPhase, t: extended; Stereo: boolean): extended;
function GetAudioSubLevel(Freq, LeftLevel, RightLevel, LeftPhase, RightPhase, Pilote19KLevel, Pilote19KPhase, Pilote38KPhase, t: extended; Stereo: boolean): extended;
function GetMPXLevel(Freq, LeftLevel, RightLevel, LeftPhase, RightPhase, Pilote19KLevel, Pilote19KPhase, Pilote38KPhase, t: extended; Stereo: boolean): extended;

const
  LFLevelRef = 0.775;

implementation


function IsBitSet(const i, Nth: integer): boolean;
begin
  // returns True if a bit is ON (1)
  // Nth can have any bit order value in [0..31]
  Result := (i and (1 shl Nth)) <> 0;
end;

function BitToOn(const i, Nth: integer): integer;
begin
  if not IsBitSet(i, Nth) then
    Result := i or (1 shl Nth)
  else
    Result := i;
end;

function BitToOff(const i, Nth: integer): integer;
begin
  if IsBitSet(i, Nth) then
    Result := i and ((1 shl Nth) xor $FFFFFFFF)
  else
    Result := i;
end;

function BitSwitch(const i, Nth: integer): integer;
begin
  // reverse the state of a bit
  Result := i xor (1 shl Nth);
end;

function ReverseAllBits(const i: integer): integer;
var
  N: integer;
begin
  // has the same effect as the ~ (one's complement) operator in C
  Result := i;
  for N := 0 to 31 do
    Result := Result xor (1 shl N);
end;

function Root(base, exponent: extended): extended;
var
  neg, even: boolean;
  r: integer;
begin
  even := False;
  r := trunc(exponent);
  neg := base < 0;
  if base < 0 then
  begin
    even := (r mod 2) = 0;
    If not even then
      base := Abs(base);
  end;
  If neg and (not even) then
    root := -Power(base, 1 / exponent)
  else
    root := Power(base, 1 / exponent);
end;

function Log(X: Extended): Extended;
const
  Base = 10;
begin
  // start modified 12/07/2002
  try
    if X > 0 then
      result := math.Log10(x)
    else
      Result := 0;
  except
    result := 0;
  end;

  exit;
  // end modified 12/07/2002

  try
    if X > 0 then
      result := ln(X) / ln(base)
    else
      Result := 0;
  except
    result := 0;
  end;
end;

{
function Log10(X: Extended): Extended;
  // Log.10(X) := Log.2(X) * Log.10(2)
asm
  FLDLG2
  FLD     X
  FYL2X
  FWAIT
end;

function Log2(X: Extended): Extended;
asm
  FLD1
  FLD     X
  FYL2X
  FWAIT
end;

function LogN(Base, X: Extended): Extended;
  // Log.N(X) := Log.2(X) / Log.2(N)
asm
  FLD1
  FLD     X
  FYL2X
  FLD1
  FLD     Base
  FYL2X
  FDIV
  FWAIT
end;
}

function FactorialX (A: Cardinal): Extended;
var
  I: Integer;
begin
  if A  > 1547 then
  begin
    Result := 0.0;
    Exit;
  end;
  Result := 1.0;
  for I := 2 to A do
    Result := Result * I;
end;

Function DegToRad(X: Extended): Extended;
begin
  try
    result := (X * pi) / 180;
  except
    result := 0;
  end;
end;

function VppToDBu(r: extended): extended;
begin
  try
    //result := 20 * log(r / 0.775);   // normally not used
    //result := 20 * log10(r / ((2 * sqrt(2)) / sqrt(0.6)))
    result := 20 * Log((r / (2 * sqrt(2)) / sqrt(0.6))); // 24/07/2001
  except
    result := 0;
  end;
end;

function VppToVrms(r: extended): extended;
begin
  try
    result := r / (2 * sqrt(2));
  except
    result := 0;
  end;
end;

function VppToDBm(r: extended): extended;
begin
  try
    // to complete
  except
    result := 0;
  end;
end;

function VppToDev(r: extended): extended;
var
  Val1, Val2: extended;
begin
  // based on 75KHz / 12dBu
  try
    Val1 := dBuToVrms(12.0);
    Val2 := VppToVrms(r);
    result := 75 / (Val1 / Val2);
  except
    result := 0;
  end;
end;

function VrmsToDbu(r: extended): extended;
begin
  try
    result := 20 * log10(r / LFLevelRef);
  except
    result := 0;
  end;
end;

function VrmsToVpp(r: extended): extended;
begin
  try
    result := r * 2 * sqrt(2);
  except
    result := 0;
  end;
end;

function VrmsToDBm(r: extended): extended;
begin
  try
    // to complete
  except
    result := 0;
  end;
end;

function VrmsToDev(r: extended): extended;
var
  Val1, Val2: extended;
begin
  // based on 75KHz / 12dBu
  try
    Val1 := dBuToVrms(12.0);
    Val2 := r;
    result := 75 / (Val1 / Val2);
  except
    result := 0;
  end;
end;

function DBuToVpp(r: extended): extended;
begin
  try
    result := 2 * sqrt(2) * sqrt(0.6) * exp((r * ln(10)) / 20);
  except
    result := 0;
  end;
end;

function DBuToVrms(r: extended): extended;
begin
  try
    result := sqrt(0.6) * exp((r * ln(10)) / 20);
  except
    result := 0;
  end;
end;

function DBuToDev(r: extended): extended;
var
  Val1, Val2: extended;
begin
  // based on 75KHz / 12dBu
  try
    Val1 := dBuToVrms(12.0);
    Val2 := dBuToVrms(r);
    result := 75 / (Val1 / Val2);
  except
    result := 0;
  end;
end;

function VppToPower(r: extended; Imp: integer): extended;
var
  x: extended;
begin
  try
    x := r / (2 * sqrt(2));
    result := sqr(x) / imp;
  except
    result := 0;
  end;
end;

function VrmsToPower(r: extended; Imp: integer): extended;
begin
  try
    result := sqr(r) / imp;
  except
    result := 0;
  end;
end;

function DevToVpp(r: extended): extended;
var
  Val1, Val2: extended;
begin
  // based on 75KHz / 12dBu
  try
    Val1 := dBuToVrms(12.0);
    Val2 := 75 / r;
    result := VrmsToVpp(Val1 / Val2);
  except
    result := 0;
  end;
end;

function DevToVrms(r: extended): extended;
var
  Val1, Val2: extended;
begin
  // based on 75KHz / 12dBu
  try
    Val1 := dBuToVrms(12.0);
    Val2 := 75 / r;
    result := Val1 / Val2;
  except
    result := 0;
  end;
end;

function DevToDBu(r: extended): extended;
var
  Val1, Val2: extended;
begin
  // based on 75KHz / 12dBu
  try
    Val1 := dBuToVrms(12.0);
    Val2 := 75 / r;
    result := VrmsToDBu(Val1 / Val2);
  except
    result := 0;
  end;
end;

function PowerToVpp(r: extended; Imp: integer): extended;
begin
  try
    result := sqrt(r * imp) * 2 * sqrt(2);
  except
    result := 0;
  end;
end;

function PowerToVrms(r: extended; Imp: integer): extended;
begin
  try
    result := sqrt(r * imp);
  except
    result := 0;
  end;
end;

function PowerToDBm(r: extended; Imp: integer): extended;
begin
  try
    // to complete
  except
    result := 0;
  end;
end;

function DBmToPower(ref, r: extended): extended;
begin
  // to confirm, not really tested
  try
    result := power(10, r / 10) * ref;
    // result := exp(1, r / 10) {* 0.001};
  except
    result := 0;
  end;
end;

function DBmToVpp(r: extended): extended;
begin
  // to confirm, not tested
  try
    result := 2 * sqrt(2) * sqrt(0.05) * exp((r * ln(10)) / 20);
  except
    result := 0;
  end;
end;

function DBmToVrms(r: extended): extended;
begin
  // to confirm, not tested
  try
    result := sqrt(0.05) * exp((r * ln(10)) / 20);
  except
    result := 0;
  end;
end;

function PreEmphasize(Ve, freq, RC: extended; Preemp: boolean): extended;
var
  Fc: extended;
begin
  result := Ve;
  if not Preemp then exit;
  if RC = -1 then exit;
  try
    Fc := 1 / (2 * Pi * RC);
    Result := Ve * sqrt(1 + (sqr(freq / Fc)));
  except
    result := 0.0;
  end;
end;

function DeEmphasize(Ve, freq, RC: extended; Preemp: boolean): extended;
var
  Fc: extended;
begin
  result := Ve;
  if not Preemp then exit;
  if RC = -1 then exit;
  try
    Fc := 1 / (2 * Pi * RC);
    Result := Ve / (sqrt(1 + (sqr(freq / Fc))));
  except
    result := 0.0;
  end;
end;

// Old GetMPXMainLevel function version, not take care about time of measure
// Only take care aboute phase between left and right signals
function GetMPXMainLevel(LeftLevel, RightLevel, Phase: extended): extended;
var
  phiM: extended;
begin
  if (phase = 0) and (LeftLevel = RightLevel) then begin
    result := (LeftLevel + RightLevel) / 2;
  end else if (phase = 0) and (LeftLevel <> RightLevel) then begin
    phiM := arcTan(0 / (LeftLevel + RightLevel));
    result := (abs(LeftLevel + RightLevel) / cos(DegToRad(phiM))) / 2;
  end else if ((phase = 180) or (phase = -180)) and (LeftLevel = RightLevel) then begin
    result := 0;
  end else if ((phase = 180) or (phase = -180)) and (LeftLevel <> RightLevel) then begin
    phiM := arcTan((RightLevel * Sin(DegToRad(phase))) / (LeftLevel + (RightLevel * Cos(DegToRad(phase)))));
    result := (abs(LeftLevel + (RightLevel * cos(DegToRad(phase)))) / cos(DegToRad(phiM))) / 2;
  end else if ((phase <> 180) and (phase <> -180) and (phase <> 0)) and (LeftLevel <> RightLevel) then begin
    phiM := arcTan((RightLevel * Sin(DegToRad(phase))) / (LeftLevel + (RightLevel * Cos(DegToRad(phase)))));
    result := (abs(LeftLevel + (RightLevel * cos(DegToRad(phase)))) / cos(DegToRad(phiM))) / 2;
  end else if ((phase <> 180) and (phase <> -180) and (phase <> 0)) and (LeftLevel = RightLevel) then begin
    phiM := arcTan((RightLevel * Sin(DegToRad(phase))) / (LeftLevel + (RightLevel * Cos(DegToRad(phase)))));
    result := (abs(LeftLevel + (RightLevel * cos(DegToRad(phase)))) / cos(DegToRad(phiM))) / 2;
  end;
  result := abs(result);
end;

// Old GetMPXSubLevel function version, not take care about time of measure
// Only take care aboute phase between left and right signals
function GetMPXSubLevel(LeftLevel, RightLevel, Phase: extended): extended;
var
  phiS: extended;
begin
  if (phase = 0) and (LeftLevel = RightLevel) then begin
    result := 0;
  end else if (phase = 0) and (LeftLevel <> RightLevel) then begin
    phiS := arcTan(0);
    result := (abs(LeftLevel - (RightLevel * cos(0))) / cos(DegToRad(phiS))) / 2;
  end else if ((phase = 180) or (phase = -180)) and (LeftLevel = RightLevel) then begin
    result := (LeftLevel + RightLevel) / 2;
  end else if ((phase = 180) or (phase = -180)) and (LeftLevel <> RightLevel) then begin
    phiS := arcTan((RightLevel * Sin(DegToRad(phase))) / (LeftLevel - (RightLevel * Cos(DegToRad(phase)))));
    result := (abs(LeftLevel - (RightLevel * cos(DegToRad(phase)))) / cos(DegToRad(phiS))) / 2;
  end else if ((phase <> 180) and (phase <> -180) and (phase <> 0)) and (LeftLevel <> RightLevel) then begin
    phiS := arcTan((RightLevel * Sin(DegToRad(phase))) / (LeftLevel - (RightLevel * Cos(DegToRad(phase)))));
    result := (abs(LeftLevel - (RightLevel * cos(DegToRad(phase)))) / cos(DegToRad(phiS))) / 2;
  end else if ((phase <> 180) and (phase <> -180) and (phase <> 0)) and (LeftLevel = RightLevel) then begin
    phiS := arcTan((RightLevel * Sin(DegToRad(phase))) / (LeftLevel - (RightLevel * Cos(DegToRad(phase)))));
    result := (abs(LeftLevel - (RightLevel * cos(DegToRad(phase)))) / cos(DegToRad(phiS))) / 2;
  end;
  result := abs(result);
end;

function GetAudioMainLevel(Freq, LeftLevel, RightLevel, LeftPhase, RightPhase, Pilote19KLevel, Pilote19KPhase, Pilote38KPhase, t: extended; Stereo: boolean): extended;
var
  tmpA, tmpB, tmp19, tmp38,
  PhiA, PhiB, Phi19, Phi38,
  Main, Sub,
  amp: extended;
begin
  phiA := LeftPhase;
  try
    PhiB := (2 * pi * RightPhase) / 360;
  except
    PhiB := 0;
  end;
  Phi19 := Pilote19KPhase;
  try
    Phi38 := (2 * pi * Pilote38KPhase) / 360;
  except
    PhiB := 0;
  end;
  tmpA := LeftLevel * sin((2 * pi * freq * t) + phiA);
  tmpB := RightLevel * sin((2 * pi * freq * t) + phiB);
  tmp38 := sin((2 * pi * 38000 * t) + phi38);
  tmp19 := Pilote19KLevel * sin((2 * pi * 19000 * t) + phi19);

  // Niveau de (A + B)
  Main := (tmpA + tmpB) / 2;

  result := Main;
end;

function GetAudioSubLevel(Freq, LeftLevel, RightLevel, LeftPhase, RightPhase, Pilote19KLevel, Pilote19KPhase, Pilote38KPhase, t: extended; Stereo: boolean): extended;
var
  tmpA, tmpB, tmp19, tmp38,
  PhiA, PhiB, Phi19, Phi38,
  Main, Sub,
  amp: extended;
begin
  phiA := LeftPhase;
  try
    PhiB := (2 * pi * RightPhase) / 360;
  except
    PhiB := 0;
  end;
  Phi19 := Pilote19KPhase;
  try
    Phi38 := (2 * pi * Pilote38KPhase) / 360;
  except
    PhiB := 0;
  end;
  tmpA := LeftLevel * sin((2 * pi * freq * t) + phiA);
  tmpB := RightLevel * sin((2 * pi * freq * t) + phiB);
  tmp38 := sin((2 * pi * 38000 * t) + phi38);
  tmp19 := Pilote19KLevel * sin((2 * pi * 19000 * t) + phi19);

  // Niveau de (A - B)
  Sub := ((tmpA - tmpB) / 2) * tmp38;

  // Niveau du MPX
  if stereo then
    result := Sub
  else
    result := 0;
end;

function GetMPXLevel(Freq, LeftLevel, RightLevel, LeftPhase, RightPhase, Pilote19KLevel, Pilote19KPhase, Pilote38KPhase, t: extended; Stereo: boolean): extended;
var
  tmpA, tmpB, tmp19, tmp38,
  PhiA, PhiB, Phi19, Phi38,
  Main, Sub,
  amp: extended;
begin
  phiA := LeftPhase;
  try
    PhiB := (2 * pi * RightPhase) / 360;
  except
    PhiB := 0;
  end;
  Phi19 := Pilote19KPhase;
  try
    Phi38 := (2 * pi * Pilote38KPhase) / 360;
  except
    PhiB := 0;
  end;
  tmpA := LeftLevel * sin((2 * pi * freq * t) + PhiA);
  tmpB := RightLevel * sin((2 * pi * freq * t) + PhiB);
  tmp19 := Pilote19KLevel * sin((2 * pi * 19000 * t) + Phi19);
  tmp38 := sin((2 * pi * 38000 * t) + Phi38);

  // (A + B) level
  Main := (tmpA + tmpB) / 2;

  // (A - B) level
  Sub := ((tmpA - tmpB) / 2) * tmp38;

  // MPX level at "t" time
  if stereo then
    amp := Main + Sub + tmp19
  else
    amp := main;

  result := amp;
end;


end.
