unit rmMisc;

interface

uses
  strCon,
  windows, forms, sysUtils, Dialogs, classes, ShellAPI, Printers,
  Graphics, ClipBrd, IniFiles, DBTables
  //, DDEMan
  ;

resourceString

  // misc strings
  EuroChar = '';
  FrancsChar = 'F';


// files and drives routines
function  CopyFile(Src, Dest: TFileName): boolean;
function  FastFileCopy(const InFileName, outFileName: string): boolean;
function  DeleteFileWithUndo(sFileName: string): boolean;
function  FileDateTime(const FileName: string): TDateTime;
function  GetFileSize(const FileName: string): LongInt;
function  GetFileSizeStr(i: cardinal): string;
function  GetFileDate(const FileName: string): TDateTime;
function  HasAttr(const FileName: string; Attr: Word): Boolean;
function  MakeDir(dir: string): boolean;
function  MakePath(path: string): boolean;
function  GetWinDir: string;
function  GetSysDir: string;
function  CheckBackSlash(s: string; SlashAtBegin, SlashAtEnd: boolean): string;
function  GetFullFileName(sPath, sFile: string): TFileName;
function  DiskInDrive(drive: Char): boolean;
function  IsCDAudio(DriveChar: Char): string;
function  IniMoveSection(sFile, sSection: string; iOffset: integer): boolean;

// system routines
procedure Delay(ms: Longword; idle: boolean);
function  ExecuteFile(const FileName, Params, DefaultDir: string; ShowCmd: Integer): THandle;
function  GetWin32Plateform: string;
function  GetDrivesList(var list: TStringList): integer;
// Procedure Keyb_Event; Far; External 'USER' index 289;
// Procedure SnapShot; assembler;
procedure  DesktopCopyToClipboard(x, y, w, h: integer);
//procedure ActiveWindowCopyToClipboard;

// Date / Time routines
function  DateIsValid(s: string): boolean;
function  TimeIsValid(s: string): boolean;
procedure GetBeginEndOfWeek(day: TDateTime; var beginDate: TDateTime; var EndDate: TDateTime);
function  GetWeekNum(day: TDateTime): byte;

// Database routines
function  DB_RenameFieldName(Tbl: TTable; sOldFieldName, sNewFieldName: string; AcceptEmptyString: boolean): boolean;
function  DB_ResizeFieldLength(Tbl: TTable; sFieldName: string;  iNewSize: integer; AcceptEmptyString: boolean): boolean;
function  DB_RenameFieldContent(Tbl: TTable; sFieldName, sOldContent, sNewContent: string; AcceptEmptyString: boolean): integer;

// string routines
function  ReverseStr(S: string): string;
function  Shorten(S: string; Cut: Integer): string;
function  StrFixLength(s, position: string; ch: char; len: word): string;
procedure PrintStrings(Strings: TStrings);
procedure GetGoodDecimalSeparator(var s: string);
function  CleanString(s: string): string;
function  CharsFoundInString(
            s: string;
            ArrChWaited: array of Char;
            ArrChPos: array of byte): boolean;
function  StrToList(s, c: string; var lst: TStringList; ClearList: boolean): integer;
function  StrToKeyAndValue(s, c: string; var sKey: string; var sVal: string): boolean;
function  StrToKeyAndValueEx(s, c, sKey1, sKey2: string; var sVal1, sVal2: string): boolean;
function  ExtractPartOfList(var lst: TStringList; sStart, sStop: string): integer;

// misc routines
//procedure GotoURLWithNetscape(sURL: string; NetscapeLocalisation: string);

// Conversion routines
function  StrToHex(s: string; bInsertSpace: boolean): string;
function  Str2Int(s: string): integer;   // with Try..except
function  Int2Str(i: integer): string ;  // with Try..except
function  StrToBool(s: string): boolean ;
function  CurrStrToFloatStr(s: string): string;
function  EuroStrToFrancsStr(s: string; Ratio: extended; UseCurStr: boolean): string;
function  FrancsStrToEuroStr(s: string; Ratio: extended; UseCurStr: boolean): string;
function  EuroStrToFrancs(s: string; Ratio: extended): currency;
function  FrancsStrToEuro(s: string; Ratio: extended): currency;
function  EuroToFrancs(cur: currency; Ratio: extended): currency;
function  FrancsToEuro(cur: currency; Ratio: extended): currency;
function  GetRequiredBits(MaxVal: integer): integer;
function  RealToStr(nb: byte; r: double): string;
function  StrToReal(s: string): double;
procedure StrToValueAndUnit(s: string; var rValue: real; var sUnit: string; KeepUnitOnError: boolean);
function  IntToBits(i: integer; nBits: byte): string;
function  ByteToBits(b: byte; nBits: byte): string;
function  WordToBits(w: word; nBits: byte): string;
function  LongWordToBits(lw: LongWord; nBits: byte): string;
function  BitsToByte(s: string): byte;
function  BitsToWord(s: string): word;
function  BitsToLongWord(s: string): LongWord;
function  BoolToStr(val: boolean; sTrue, sFalse: string): string;
function  BoolToColor(val: boolean; clTrue, clFalse: TColor): TColor;
function  BoolToByte(val: boolean; bTrue, bFalse: byte): byte;
function  BoolToInt(val: boolean; iTrue, iFalse: integer): integer;
function  ByteToHex(val: byte): string;
function  WordToHex(val: Word): string;
function  LongWordToHex(val: Longword): string;
function  HexToByte(s: string): byte;
function  HexToWord(s: string): word;
function  HexToLongWord(s: string): LongWord;
function  ArrayBoolToInteger(Ar: array of boolean): integer;
function  IntegerToArrayBool(iVal: integer; var Ar: array of boolean): boolean;

implementation

{
**********************************************************************
*  files and drives routines
**********************************************************************
}

function CopyFile(Src, Dest: TFileName): boolean;
var
  FromF, ToF: file;
  NumRead, NumWritten: Integer;
  Buf: array[1..2048] of Char;
  SrcHandle, DestHandle: word;
  Err: boolean;
begin
  result := false;
  Err := false;

  if AnsiUpperCase(Src) = AnsiUpperCase(Dest) then
    exit;

  // file copy
  try
    AssignFile(FromF, Src);
    Reset(FromF, 1);
    AssignFile(ToF, Dest);
    Rewrite(ToF, 1);
    repeat
      BlockRead(FromF, Buf, SizeOf(Buf), NumRead);
      BlockWrite(ToF, Buf, NumRead, NumWritten);
    until (NumRead = 0) or (NumWritten <> NumRead);
    try
      FileSetDate(TFileRec(ToF).handle, FileGetDate(TFileRec(FromF).handle));
    except
      Err := true;
    end;
    CloseFile(FromF);
    CloseFile(ToF);
    result := true;
  except
    Exit;
  end;

  // attributs copy
  try
    FileSetAttr(Dest, FileGetAttr(src));
  except
    Err := true;
  end;

  result := not Err;

  exit;

  // date copy
  try
    srcHandle := fileOpen(src, fmOutput);
    DestHandle := FileOpen(dest, fmInput);
    showmessage(src + #13 + inttostr(srchandle));
    FileSetDate(DestHandle, FileGetDate(SrcHandle));
    fileClose(srcHandle);
    fileClose(destHandle);
  except
    Err := true;
  end;
  result := not Err;
end;

function FastFileCopy(const InFileName, outFileName: string): boolean;
const
  BufSize = 3 * 4 * 4096; // 48K
type
  PBuffer = ^TBuffer;
  TBuffer = array[1..BufSize] of byte;
var
  size: integer;
  buffer: PBuffer;
  inFile, outFile: file;
  //sizeDone,
  //SizeFile: integer;
begin
  result := false;
  if inFileName = outFileName then exit;
  buffer := nil;
  assign(inFile, inFileName);
  system.Reset(inFile, 1);
  try
    //sizeFile := FileSize(inFile);
    assign(outFile, outFileName);
    system.rewrite(outFile, 1);
    try
      //sizeDone := 0;
      new(buffer);
      repeat
        blockRead(inFile, buffer^, bufSize, size);
        //inc(sizeDone, size);
        // callBack(sizeDone, sizeFile);
        blockWrite(outFile, buffer^, size);
      until size < bufSize;
      FileSetDate(TFileRec(outFile).handle, FileGetDate(TFileRec(inFile).handle));
      result := true;
    finally
      if buffer <> nil then
        dispose(buffer);
      system.close(outFile);
    end;
  finally
    system.close(inFile);
  end;
end;

function DeleteFileWithUndo(sFileName: string): boolean;
var
  fos : TSHFileOpStruct;
begin
  FillChar( fos, SizeOf( fos ), 0 );
  with fos do begin
    wFunc  := FO_DELETE;
    pFrom  := PChar( sFileName );
    fFlags := FOF_ALLOWUNDO or FOF_NOCONFIRMATION or FOF_SILENT;
  end;
  Result := ( 0 = ShFileOperation( fos ) );
end;

function FileDateTime(const FileName: string): System.TDateTime;
var
  SearchRec: TSearchRec;
  ModifiedTime: TFileTime;
  SystemTime: TSystemTime;
begin
  Result := 0;
  try
    If FindFirst(FileName, faAnyFile, SearchRec) = 0 then
    begin
      ModifiedTime := SearchRec.FindData.ftLastWriteTime;
      FileTimeToLocalFileTime(ModifiedTime, ModifiedTime);
      FileTimeToSystemTime(ModifiedTime, SystemTime);
      Result := SystemTimeToDateTime(SystemTime);
    end;
  finally
    FindClose(SearchRec);
  end;
  {
  Result := FileDateToDateTime(FileAge(FileName));
  }
end;

function GetFileSize(const FileName: string): LongInt;
var
  SearchRec: TSearchRec;
begin
  if not fileExists(FileName) then
  begin
    result := -1;
    exit;
  end;
  try
    if FindFirst(ExpandFileName(FileName), faAnyFile, SearchRec) = 0 then
      Result := SearchRec.Size
    else
      Result := -1;
  except
    Result := -1;
  end;
end;

function GetFileDate(const FileName: string): TDateTime;
begin
  if not fileExists(FileName) then begin
    result := 0;
    exit;
  end;
  try
    result := FileDateToDateTime(fileAge(FileName));
  except
    result := 0;
  end;
end;

function  GetFileSizeStr(i: cardinal): string;
begin
  case i of
    0..1024 : result := intToStr(i) + ' octets';
    1025..1048576 : result := RealToStr(2, (i / 1024)) + ' KO';
    else result := RealToStr(2, (i / 1048576)) + ' MO';
  end;
end;

function HasAttr(const FileName: string; Attr: Word): Boolean;
begin
  Result := (FileGetAttr(FileName) and Attr) = Attr;
end;

function MakeDir(dir: string): boolean;
var
  rech : TSearchRec;
begin
  result := true;
  if (length(dir) = 2) and (dir[1] in ['a'..'z', 'A'..'Z']) and (dir[2] = ':') then exit;
  {$I-}
  if findFirst(dir + '\*.*', faDirectory, Rech) <> 0 then
  begin
    mkDir(dir);
    if IOResult <> 0 then
    begin
      showMessage('Impossible de crer le rpertoire "' + dir + '"');
      result := false;
    end;
  end;
  findClose(rech);
  {$I+}
end;

function MakePath(path: string): boolean;
var
  i : integer;
  s : string;
  OK: boolean;
begin
  try
    s := '';
    for i := 1 to length(path) do
    begin
      s := s + path[i];
      if i < length(path) then
        if path[i + 1] = '\' then
        begin
          OK := MakeDir(s);
          if not OK then
            break;
        end;
    end;
    OK := MakeDir(path);
    result := OK;
  except
    result := false;
  end;
end;

function GetWinDir: string;
var
  wndDir: array[0..200] of char;
  windowsDir: string;
  retRslt: word;
begin
  retRslt := GetWindowsDirectory(wndDir, sizeOf(wndDir));
  if (retRslt > 0) then windowsDir := strPas(wndDir);
  if windowsDir[length(windowsDir) - 1] <> '\' then
    windowsDir := windowsDir + '\';
  result := windowsDir;
end;

function GetSysDir: string;
var
  sysDir: array[0..200] of char;
  systemDir: string;
  retRslt: word;
begin
  retRslt := GetSystemDirectory(sysDir, sizeOf(sysDir));
  if (retRslt > 0) then systemDir := strPas(sysDir);
  if systemDir[length(systemDir) - 1] <> '\' then
    systemDir := systemDir + '\';
  result := systemDir;
end;

function CheckBackSlash(s: string; SlashAtBegin, SlashAtEnd: boolean): string;
var
  sTmp: string;
begin
  if s = '' then begin
    result := '';
    exit;
  end;
  sTmp := s;
  if (copy(sTmp, 1, 1) = '\') and (not SlashAtBegin) then
    delete(sTmp, 1, 1);
  if (copy(sTmp, 1, 1) <> '\') and SlashAtBegin then
    sTmp := '\' + sTmp;
  if (copy(sTmp, length(sTmp), 1) = '\') and (not SlashAtEnd) then
    delete(sTmp, length(sTmp), 1);
  if (copy(sTmp, length(sTmp), 1) <> '\') and SlashAtEnd then
    sTmp := sTmp + '\';
  result := sTmp;
end;

function GetFullFileName(sPath, sFile: string): TFileName;
var
  tmpPath, tmpFile: string;
begin
  tmpPath := sPath;
  tmpFile := sFile;
  if copy(tmpPath, length(tmpPath), 1) <> '\' then
    tmpPath := tmpPath + '\';
  if copy(tmpFile, 1, 1) = '\' then
    delete(tmpFile, 1, 1);
  result := tmpPath + tmpFile;
end;

function DiskInDrive(drive: Char): boolean;
var
  errorMode: word;
begin
  if drive in ['a'..'z'] then dec(drive, $20); {met en majuscule}
  if not (drive in ['A'..'Z']) then
    raise EConvertError.create('Lettre de lecteur invalide');
  errorMode := setErrorMode(SEM_FailCriticalErrors);
  try
    result := not(DiskSize(ord(drive) - $40) = -1);
  finally
    setErrorMode(errorMode);
  end;
end;

function IsCDAudio(DriveChar: Char): string;
var
  SearchMask: string[7];
  SearchRec: TSearchRec;
  DotPos: Byte;
  OldErrorMode: Word;
begin
  OldErrorMode := SetErrorMode(SEM_FAILCRITICALERRORS);
  try
    SearchMask := 'c:\*.*';
    SearchMask[1] := DriveChar;
    if FindFirst(SearchMask, faVolumeID, SearchRec) = 0 then
    begin
      Result := SearchRec.Name;
      DotPos := Pos('.', Result);
      if DotPos <> 0 then
        System.Delete(Result, DotPos, 1);
      if DriveChar < 'a' then
        Result := AnsiUpperCase(Result)
      else
        Result := AnsiLowerCase(Result)
    end
    else Result := 'Aucun CD audio';
    if Result <> 'AUDIO CD' then result := 'Aucun CD audio';
  finally
    SetErrorMode(OldErrorMode);
  end;
end;

function  IniMoveSection(sFile, sSection: string; iOffset: integer): boolean;
var
  SectionList: TStringList;
  Section: string;
  i: integer;
  ini: TIniFile;
begin
  // result := false;
  SectionList := TStringList.Create;
  Ini := TIniFile.Create(sFile);
  try
    ini.ReadSections(SectionList);
    for i := 0 to SectionList.Count - 1 do begin
      section := SectionList[i];

      // not finished

    end;
  finally
    SectionList.Free;
    ini.free;
  end;
  result := true;
end;


{
**********************************************************************
*  system routines
**********************************************************************
}

procedure Delay(ms: Longword; idle: boolean);
var
  t: Longword;
begin
  t := getTickCount;
  repeat
    if idle then
      application.processMessages;
  until (getTickCount - t) > ms;
end;

function ExecuteFile(const FileName, Params, DefaultDir: string;
  ShowCmd: Integer): THandle;
var
  zFileName, zParams, zDir: array[0..79] of Char;
begin
  Result := ShellExecute(Application.MainForm.Handle, nil,
    StrPCopy(zFileName, FileName), StrPCopy(zParams, Params),
    StrPCopy(zDir, DefaultDir), ShowCmd);
end;

function GetWin32Plateform: string;
begin
  case Win32Platform of
    VER_PLATFORM_WIN32s        : result := 'Win 32s';
    VER_PLATFORM_WIN32_WINDOWS : result := 'Win 95';
    VER_PLATFORM_WIN32_NT      : result := 'Win NT';
  end;
end;

function GetDrivesList(var list: TStringList): integer;
var
  buffer: array[0..500] of Char;
  TmpPC: PChar;
  Typ: integer;
  cnt: byte;
begin
  cnt := 0;
  GetLogicalDriveStrings(sizeOf(buffer), buffer);
  TmpPC := Buffer;
  while TmpPC[0] <> #0 do begin
    Typ := GetDriveType(TmpPC);
    case typ of
      DRIVE_REMOVABLE : list.Add(TmpPC + ' [Removable]');
      DRIVE_FIXED : list.Add(TmpPC + ' [Fixe]');
      DRIVE_REMOTE : list.Add(TmpPC + ' [Remote]');
      DRIVE_CDROM : list.Add(TmpPC + ' [CD-ROM]');
      DRIVE_RAMDISK : list.Add(TmpPC + ' [RamDisk]');
    else
      list.Add(TmpPC + ' [Unknown]');
    end;
    TmpPC := strEnd(TmpPC) + 1;
    inc(cnt);
  end;
  result := cnt;
end;

{
Procedure SnapShot; assembler;
asm
  mov ax,VK_MENU
  sub bx,bx
  call keyb_event
  mov ax,VK_SNAPSHOT
  sub bx,bx
  call keyb_event
  mov ax,VK_SNAPSHOT+$8000
  sub bx,bx
  call Keyb_Event
  mov ax,VK_MENU+$8000
  sub bx,bx
  call Keyb_Event
end;
}

procedure DeskTopCopyToClipboard(x, y, w, h: integer);
var
  ScreenDC: HDC;
  ScreenHandle: HWND;
  bmp: TBitmap;
begin
  ScreenHandle := GetDesktopWindow;
  ScreenDC := GetDC(ScreenHandle);
  try
    bmp := TBitmap.Create;
    try
      bmp.Width := w;
      bmp.Height := h;
      BitBlt(bmp.Canvas.Handle, x, y, w, h, ScreenDC, 0, 0, SRCCOPY);
      // put in a TImage object ---> Image.Picture.Bitmap:=bmp
      Clipboard.Assign(bmp);
    finally
      bmp.free;
    end;
  finally
    ReleaseDC(ScreenHandle,ScreenDC);
  end;
end;

{
procedure Keybd_Event; far; external 'USER' index 289;
procedure ActiveWindowCopyToClipboard; assembler;
asm
  mov ax,VK_MENU
  sub bx,bx
  call Keybd_Event
  mov ax,VK_SNAPSHOT
  sub bx,bx
  call Keybd_Event
  mov ax,VK_SNAPSHOT+$8000
  sub bx,bx
  call Keybd_Event
  mov ax,VK_MENU+$8000
  sub bx,bx
  call Keybd_Event
end;
}


{
**********************************************************************
*  Date / Time routines
**********************************************************************
}

function DateIsValid(s: string): boolean;
begin
  try
    strToDate(s);
    result := true;
  except
    result := false;
  end;
end;

function TimeIsValid(s: string): boolean;
begin
  try
    strToTime(s);
    result := true;
  except
    result := false;
  end;
end;

procedure GetBeginEndOfWeek(day: TDateTime; var beginDate: TDateTime; var EndDate: TDateTime);
var
  TheDay: integer;
begin
  TheDay := DayOfWeek(day);
  case TheDay of
    1 : BeginDate := day - 6; // Dimanche
    2 : BeginDate := day;     // lundi
    3 : BeginDate := day - 1; // mardi
    4 : BeginDate := day - 2; // mercredi
    5 : BeginDate := day - 3; // jeudi
    6 : BeginDate := day - 4; // vendredi
    7 : BeginDate := day - 5; // samedi
  end;
  endDate := beginDate + 6;
end;

function GetWeekNum(day: TDateTime): byte;
var
  jour, mois, annee, numSemaines: word;
  i: integer;
  beginDate, endDate: TDateTime;
  numJours: integer;
begin
  numSemaines := 0;
  decodeDate(day, annee, mois, jour);
  beginDate := trunc(encodeDate(annee, 1, 1));
  endDate := trunc(day);
  numJours := trunc(endDate - beginDate);
  for i := 0 to numJours do begin
    if (sysUtils.dayOfWeek(beginDate + i) = 2) then
      inc(numSemaines);
  end;
  {
  if (sysUtils.dayOfWeek(encodeDate(annee, 1, 1)) = 2) then
    dec(numSemaines);
  }
  result := numSemaines;
end;


{
**********************************************************************
*  Database routines
**********************************************************************
}

function  DB_RenameFieldName(Tbl: TTable; sOldFieldName, sNewFieldName: string; AcceptEmptyString: boolean): boolean;
var
  i: Integer;
  sFieldName: string;
  Found: boolean;
begin
  result := false;
  found := false;
  if sOldFieldName = sNewFieldName then exit;

  // search field position in table
  for i := 0 to Tbl.FieldCount - 1 do begin
    sFieldName := Tbl.Fields[i].FieldName;
    if sFieldName = sOldFieldName then begin
      found := true;
      break;
    end;
  end;

  if not found then exit;

  // try to rename field
  with Tbl do begin
    try
      Tbl.Fields[i].FieldName := sNewFieldName;
    except
      exit;
    end;
  end;

  result := true;
end;

function  DB_ResizeFieldLength(Tbl: TTable; sFieldName: string;  iNewSize: integer; AcceptEmptyString: boolean): boolean;
begin
  result := false;
end;

function  DB_RenameFieldContent(Tbl: TTable; sFieldName, sOldContent, sNewContent: string; AcceptEmptyString: boolean): integer;
var
  sContent: string;
  i: integer;
begin
  i := 0;
  result := i;
  if sOldContent = sNewContent then exit;
  if (not AcceptEmptyString) and (sNewContent = '') then exit;

  with Tbl do begin
    try
      tbl.DisableControls;
      first;
      repeat
        sContent := fieldByName(sFieldName).asString;
        if sContent = sOldContent then begin
          edit;
          fieldByName(sFieldName).asString := sNewContent;
          try
            post;
            inc(i);
            first;
          except
            cancel;
          end;
        end;
        next;
      until EOF;
      sContent := fieldByName(sFieldName).asString;
      if sContent = sOldContent then begin
        edit;
        fieldByName(sFieldName).asString := sNewContent;
        try
          post;
        except
          cancel;
        end;
      end;
    finally
      tbl.EnableControls;
    end;
  end;
  result := i;
end;


{
**********************************************************************
*  Misc routines
**********************************************************************
}

function ReverseStr(S: string): string;
var
  Len: Integer;
  Temp: String;
  i,j: Integer;
begin
  Len := Length(S);
  j := Len;
  for i := 1 to Len do begin
    Temp[i] := S[j];
    dec(j);
  end;
  Temp[1] := S[1];
  ReverseStr := Temp;
end;

function Shorten(S: string; Cut: Integer): string;
begin
  S[1] := Chr(Ord(S[1]) - Cut);
  Shorten := S;
end;

function StrFixLength(s, position: string; ch: char; len: word): string;
var
  sTmp: string;
begin
  sTmp := '';

  // if original string length equal waited length,
  // simply return original string
  if length(s) = len then
    sTmp := s

  // if original string length larger than waited length,
  // truncate original string and simply return it
  else if length(s) > len then
    sTmp := copy(s, 1, len)

  // if original string length smaller than waited length,
  // fill original string width specified char, and return it
  else if length(s) < len then begin
    // first method
    sTmp := s;
    if copy(UpperCase(position), 1, 1) = 'L' then
      sTmp := StringOfChar(ch, len - length(s)) + sTmp
    else if copy(UpperCase(position), 1, 1) = 'R' then
      sTmp := sTmp + StringOfChar(ch, len - length(s));

    {
    // second method
    sTmp := s;
    for i := length(s) + 1 to len do begin
      if copy(UpperCase(position), 1, 1) = 'L' then
        sTmp := ch + sTmp
      else if copy(UpperCase(position), 1, 1) = 'R' then
        sTmp := sTmp + ch;
    end;
    }
  end;

  // in all cases
  result := sTmp;
end;

procedure PrintStrings(Strings: TStrings);
var
  Prn: TextFile;
  i: word;
begin
  AssignPrn(Prn);
  try
    Rewrite(Prn);
    try
      for i := 0 to Strings.Count - 1 do
        writeln(Prn, Strings.Strings[i]);
    finally
      CloseFile(Prn);
    end;
  except
    on EInOutError do
      MessageDlg('Erreur d''impression.', mtError, [mbOk], 0);
  end;
end;

procedure GetGoodDecimalSeparator(var s: string);
var
  cBadDecimalSeparator, cGoodDecimalSeparator: char;
begin
  cGoodDecimalSeparator := DecimalSeparator;
  if cGoodDecimalSeparator = '.' then
    cBadDecimalSeparator := ','
  else if cGoodDecimalSeparator = ',' then
    cBadDecimalSeparator := '.'
  else
    cBadDecimalSeparator := '.';
  while Pos(cBadDecimalSeparator, s) > 0 do
    s[Pos(cBadDecimalSeparator, s)] := cGoodDecimalSeparator;
end;

function CleanString(s: string): string;
begin
  result := trim(s);
end;

function CharsFoundInString(
           s: string;
           ArrChWaited: array of Char;
           ArrChPos: array of byte): boolean;
var
  i: integer;
  found: boolean;
begin
  result := false;
  if high(ArrChWaited) <> high(ArrChPos) then exit;
  for i := 0 to high(ArrChWaited) do begin
    found := copy(s, ArrChPos[i], 1) = ArrChWaited[i];
    if not found then exit;
  end;
  result := true;
end;

function  StrToList(s, c: string; var lst: TStringList; ClearList: boolean): integer;
var
  sep: integer;
  sTmp, StrToAdd: string;
  Err: boolean;
begin
  if ClearList then
    lst.clear;
  sTmp := s;
  Err := false;
  while (pos(c, sTmp) > 0) do begin
    sep := pos(c, sTmp);
    StrToAdd := copy(sTmp, 1, sep - 1);
    try
      lst.add(StrToAdd);
      delete(sTmp, 1, sep);
    except
      Err := true;
      break;
    end;
  end;
  if (sTmp <> '') and (not Err) then begin
    try
      lst.add(sTmp);
    except
    end;
  end;
  result := lst.count;
end;

function StrToKeyAndValue(s, c: string; var sKey: string; var sVal: string): boolean;
var
  sep: integer;
  sTmp: string;
begin
  sTmp := s;
  sKey := '';
  sVal := '';
  sep := pos(c, sTmp);
  if sep > 0 then begin
    sKey := trim(copy(sTmp, 1, sep - 1));
    delete(sTmp, 1, sep);
    sVal := trim(sTmp);
  end;
  result := (sKey <> '') and (sVal <> '');
end;

function StrToKeyAndValueEx(s, c, sKey1, sKey2: string; var sVal1, sVal2: string): boolean;
var
  sep: integer;
  sTmp1, sTmp2: string;
begin
  try
    sep := pos(sKey2, s);
    sTmp1 := trim(copy(s, 1, sep - 1));
    sTmp2 := trim(copy(s, sep, length(s)));
    StrToKeyAndValue(sTmp1, c, sKey1, sVal1);
    StrToKeyAndValue(sTmp2, c, sKey2, sVal2);
    result := true;
  except
    result := false;
  end;
end;

function  ExtractPartOfList(var lst: TStringList; sStart, sStop: string): integer;
var
  i: integer;
  s: string;
  iStart: integer;
  iStop: integer;
begin
  result := lst.Count;
  iStart := -1;
  iStop := -1;
  // verify start and end exist
  for i := 0 to pred(lst.count) do begin
    s := trim(upperCase(lst[i]));  // case insensitive
    // showmessage(s);
    if (iStart = -1) and (s = sStart) then
      iStart := i;
    if (iStop = -1) and (s = sStop) then
      iStop := i;
    if ((iStart > -1) and (iStop > -1)) then
      break;
  end;
  if ((iStart = -1) or (iStop = -1)) then
    exit;
  if (iStart > iStop) then
    exit;

  // erase lines after part end
  for i := pred(lst.count) downTo iStop do
    lst.Delete(i);

  // erase line before part begin
  for i := 0 to iStart do
    lst.Delete(0);

  result := lst.Count;
end;

{
procedure GotoURLWithNetscape(sURL: string; NetscapeLocalisation: string);
var
  dde: TDDEClientConv;
begin
  dde := TDDEClientConv.Create(nil);
  with dde do begin
    // specify the location of netscape.exe, for example 'c:\ns32\program\netscape.exe'
    ServiceApplication := NetscapeLocalisation;
    // activate the Netscape Navigator
    SetLink( 'Netscape', 'WWW_Activate' );
    RequestData('0xFFFFFFFF');
    // go to the specified URL
    SetLink( 'Netscape', 'WWW_OpenURL' );
    RequestData(sURL+',,0xFFFFFFFF,0x3,,,' );
    // ...
    CloseLink;
  end;
  dde.Free;
end;
}

{
**********************************************************************
*  Conversion routines
**********************************************************************
}

function StrToHex(s: string; bInsertSpace: boolean): string;
var
  sTmp: string;
  i, iOrd: integer;
  c: char;
begin
  sTmp := '';
  try
    for i := 1 to length(s) do
    begin
      c := s[i];
      iOrd := ord(c);
      sTmp := sTmp + IntToHex(iOrd, 2);
      if bInsertSpace then
        sTmp := sTmp  + ' ';
    end;
    result := sTmp;
  except
    result := '';
  end;
end;

function Str2Int(s: string): integer;
var
  c: char;
  i, sep: integer;
  sTmp: string;
begin
  sTmp := s;
  sep := pos(' ', sTmp);
  if sep > 0 then
    sTmp := copy(sTmp, 1, sep - 1);

  // remove all char that are not in 0..9 interval
  for i := length(sTmp) downTo 1 do
  begin
    c := stmp[i];
    if not (c in ['0'..'9']) then
      delete(sTmp, i, 1);
  end;

  try
    result := StrToInt(sTmp);
  except
    result := 0;
  end;
end;

function Int2Str(i: integer): string ;
begin
  try
    result := IntToStr(i);
  except
    result := '';
  end;
end;

function StrToBool(s: string): boolean ;
begin
  try
    result := boolean(StrToInt(s));
  except
    result := false;
  end;
end;

function CurrStrToFloatStr(s: string): string;
var
  cur: currency;
  sTmp: string;
begin
  result := '';
  if s = '' then exit;
  sTmp := UpperCase(s);
  // get value without symbol
  while pos(CURRENCYSTRING, sTmp) > 0 do
    delete(sTmp, pos(CURRENCYSTRING, sTmp), 1);
  while pos(THOUSANDSEPARATOR, sTmp) > 0 do
    delete(sTmp, pos(THOUSANDSEPARATOR, sTmp), 1);
  while pos(FrancsChar, sTmp) > 0 do
    delete(sTmp, pos(FrancsChar, sTmp), 1);
  while pos(EuroChar, sTmp) > 0 do
    delete(sTmp, pos(EuroChar, sTmp), 1);
  while pos('E', sTmp) > 0 do
    delete(sTmp, pos('E', sTmp), 1);
  while pos(' ', sTmp) > 0 do
    delete(sTmp, pos(' ', sTmp), 1);
  while pos('_', sTmp) > 0 do
    delete(sTmp, pos('_', sTmp), 1);

  try
    GetGoodDecimalSeparator(sTmp);
    cur := strToCurr(sTmp);
  except
    cur := 0;
  end;
  result := format('%f', [cur]);
end;

function EuroStrToFrancsStr(s: string; Ratio: extended; UseCurStr: boolean): string;
var
  cur: currency;
  sTmp: string;
begin
  result := '';
  if s = '' then exit;
  try
    sTmp := s;
    rmMisc.GetGoodDecimalSeparator(sTmp);
    cur := strToCurr(sTmp);
    cur := cur * Ratio;
  except
    cur := 0;
  end;
  if UseCurStr then
    result := format('%f %s', [cur, FrancsChar])
  else
    result := format('%f', [cur]);
end;

function FrancsStrToEuroStr(s: string; Ratio: extended; UseCurStr: boolean): string;
var
  cur: currency;
  sTmp: string;
begin
  result := '';
  if s = '' then exit;
  try
    sTmp := s;
    rmMisc.GetGoodDecimalSeparator(sTmp);
    cur := strToCurr(sTmp);
    cur := cur / Ratio;
  except
    cur := 0;
  end;
  if UseCurStr then
    result := format('%f %s', [cur, EuroChar])
  else
    result := format('%f', [cur]);
end;

function EuroStrToFrancs(s: string; Ratio: extended): currency;
var
  cur: currency;
  sTmp: string;
begin
  result := 0.0;
  if s = '' then exit;
  try
    sTmp := s;
    rmMisc.GetGoodDecimalSeparator(sTmp);
    cur := strToCurr(sTmp);
    cur := cur * Ratio;
  except
    cur := 0;
  end;
  result := cur;
end;

function FrancsStrToEuro(s: string; Ratio: extended): currency;
var
  cur: currency;
  sTmp: string;
begin
  result := 0.0;
  if s = '' then exit;
  try
    sTmp := s;
    rmMisc.GetGoodDecimalSeparator(sTmp);
    cur := strToCurr(sTmp);
    cur := cur / Ratio;
  except
    cur := 0;
  end;
  result := cur;
end;

function EuroToFrancs(cur: currency; Ratio: extended): currency;
begin
  try
    result := cur * Ratio;
  except
    result := 0.0;
  end;
end;

function FrancsToEuro(cur: currency; Ratio: extended): currency;
begin
  try
    result := cur / Ratio;
  except
    result := 0.0;
  end;
end;

{
function  GetRequiredBits(MaxVal: integer): integer;
  return number of bits necessary for handle maxVal in binary notation.
  for example, if maxVal = 512, GetRequiredBits return 9
  (because we must have 9 bits for handle 512 value).
  If maxVal = 513, GetRequiredBits return 10
  (because we must have 10 bits for handle 513 value).
}
function  GetRequiredBits(MaxVal: integer): integer;
var
  r: extended;
begin
  r := ln(maxVal) / ln(2);
  if r > trunc(r) then
    result := trunc(r) + 1
  else
    result := trunc(r);
end;

function RealToStr(nb: byte; r: double): string;
var
  s : string;
begin
  try
    str(r: 0: nb, s);
  except
    s := '';
  end;
  result := s;
end;

function StrToReal(s: string): double;
var
  sTmp: string;
  reel : double;
  ErrorCode: integer;
begin
  sTmp := s;
  try
    GetGoodDecimalSeparator(sTmp);
    val(sTmp, reel, ErrorCode);
    if ErrorCode > 0 then
      val(copy(s, 1, ErrorCode -1), reel, ErrorCode);
  except
    reel := 0.0;
  end;
  result := reel;
end;

procedure StrToValueAndUnit(s: string; var rValue: real; var sUnit: string; KeepUnitOnError: boolean);
var
  sTmp, sV, sU: string;
  sep, ErrorCode: integer;
  sign: integer;
begin
  sTmp := trim(s);
  GetGoodDecimalSeparator(sTmp);

  // get sign
  if copy(sTmp, 1, 1) = '+' then begin
    sign := 1;
    delete(sTmp, 1, 1);
  end else if copy(sTmp, 1, 1) = '-' then begin
    sign := -1;
    delete(sTmp, 1, 1);
  end else begin
    sign := 1;
  end;

  sep := pos(' ', sTmp);

  // value and unit separated by a space char
  if sep > 0 then begin
    sV := copy(sTmp, 1, sep - 1);
    rValue := strToReal(sV) * sign;
    delete(sTmp, 1, sep);
    sU := copy(sTmp, 1, length(sTmp));
    sUnit := trim(sU);

  // value and unit not separated by a space char
  end else begin
    val(sTmp, rValue, ErrorCode);
    if ErrorCode > 0 then begin
      sU := copy(sTmp, ErrorCode, length(sTmp));
      sTmp := copy(sTmp, 1, ErrorCode - 1);
      val(sTmp, rValue, ErrorCode);
      rValue := rValue * sign;
      if KeepUnitOnError then
        sUnit := trim(sU)
      else
        sUnit := '';
    end;
  end;
end;

function  IntToBits(i: integer; nBits: byte): string;
begin
  result := '';
  try
    if nBits > 32 then
      nBits := 32;
    while nBits > 0 do begin
      dec(nBits);
      result := result + intToStr((i shr nBits) and 1);
    end;
  except
    result := '';
  end;
end;

function ByteToBits(b: byte; nBits: byte): string;
var
  i: integer;
  bt: byte;
  s: string;
begin
  bt := $01;
  s := '';
  try
    for i := 1 to nBits do begin
      if (b and bt) > 0 then
        s := '1' + s
      else
        s := '0' + s;
      {$R-}
      bt := bt shl 1;
      {$R+}
    end;
    result := s;
  except
    result := '';
  end;
end;

function  WordToBits(w: word; nBits: byte): string;
var
  i: integer;
  wWord: word;
  sTmp: string;
begin
  wWord := $0001;
  sTmp := '';
  try
    for i := 1 to nBits do begin
      if (w and wWord) > 0 then
        sTmp := '1' + sTmp
      else
        sTmp := '0' + sTmp;
      {$R-}
      wWord := wWord shl 1;
      {$R+}
    end;
    result := sTmp;
  except
    result := '';
  end;
end;

function  LongWordToBits(lw: LongWord; nBits: byte): string;
var
  i: integer;
  lWord: LongWord;
  sTmp: string;
begin
  lWord := $0001;
  sTmp := '';
  try
    for i := 1 to nBits do begin
      if (lw and lWord) > 0 then
        sTmp := '1' + sTmp
      else
        sTmp := '0' + sTmp;
      {$R-}
      lWord := lWord shl 1;
      {$R+}
    end;
    result := sTmp;
  except
    result := '';
  end;
end;

// convert 8 bits string (00010011) to a byte (18d)
function  BitsToByte(s: string): byte;
var
  i: integer;
  bByte, n: byte;
  sTmp: string;
begin
  try
    sTmp := s;

    // make sur length is 8 char length
    if length(sTmp) <> 8 then
    begin
      if length(sTmp) < 8 then
        sTmp := StrFixLength(sTmp, 'Left', '0', 8)
      else if length(sTmp) > 8 then
        sTmp := copy(s, 1, 8);
    end;

    bByte := 0;
    n := 1;
    for i := 8 downTo 1 do begin
      if i <> 8 then
        n := n * 2;
      if sTmp[i] = '1' then
        bByte := bByte + n;
    end;
    result := bByte;
  except
    result := 0;
  end;
end;

// convert 8 bits string to int value
function  BitsToWord(s: string): word;
var
  i: integer;
  wWord, n: word;
  sTmp: string;
begin
  try
    sTmp := s;
    if length(sTmp) <> 16 then
      sTmp := StrFixLength(sTmp, 'Left', '0', 16);

    wWord := 0;
    n := 1;
    for i := 16 downTo 1 do begin
      if i <> 16 then
        n := n * 2;
      if sTmp[i] = '1' then
        wWord := wWord + n;
    end;
    result := wWord;
  except
    result := 0;
  end;
end;

function  BitsToLongWord(s: string): LongWord;
var
  i: integer;
  lWord, n: LongWord;
  sTmp: string;
begin
  try
    sTmp := s;
    if length(sTmp) <> 32 then
      sTmp := StrFixLength(sTmp, 'Left', '0', 32);

    lWord := 0;
    n := 1;
    for i := 32 downTo 1 do begin
      if i <> 32 then
        n := n * 2;
      if sTmp[i] = '1' then
        lWord := lWord + n;
    end;
    result := lWord;
  except
    result := 0;
  end;
end;

function BoolToStr(val: boolean; sTrue, sFalse: string): string;
begin
  if val then
    result := sTrue
  else
    result := sFalse;
end;

function BoolToColor(val: boolean; clTrue, clFalse: TColor): TColor;
begin
  if val then
    result := clTrue
  else
    result := clFalse;
end;

function BoolToByte(val: boolean; bTrue, bFalse: byte): byte;
begin
  if val then
    result := bTrue
  else
    result := bFalse;
end;

function  BoolToInt(val: boolean; iTrue, iFalse: integer): integer;
begin
  if val then
    result := iTrue
  else
    result := iFalse;
end;

function ByteToHex(val: byte): string;
begin
  try
    result := intToHex(val, 2);
  except
    result := '';
  end;
end;

function WordToHex(val: Word): string;
begin
  try
    result := intToHex(val, 4);
  except
    result := '';
  end;
end;

function LongWordToHex(val: Longword): string;
begin
  try
    result := intToHex(val, 8);
  except
    result := '';
  end;
end;

function HexToByte(s: string): byte;
var
  c: char;
  sByte: string;
  i: integer;
  sTmp: string;
begin
  try
    sTmp := s;

    // make sure length is 2 chars
    if length(sTmp) <> 2 then
      sTmp := StrFixLength(sTmp, 'Left', '0', 2);

    sByte := '';
    for i := 1 to 2 do begin
      c := upcase(sTmp[i]);
      case ord(c) of
        48..57 : sByte := sByte + ByteToBits(ord(c) - 48, 4);
        65..70 : sByte := sByte + ByteToBits(ord(c) - 55, 4);
      end;
    end;
    result := BitsToByte(sByte);
  except
    result := 0;
  end;
end;

function HexToWord(s: string): word;
var
  c: char;
  sWord: string;
  i: integer;
  sTmp: string;
begin
  try
    sTmp := s;
    if copy(sTmp, 1, 1) = '$' then
      delete(sTmp, 1, 1);

    if length(sTmp) <> 4 then
      sTmp := StrFixLength(sTmp, 'Left', '0', 4);

    sWord := '';
    for i := 1 to 4 do begin
      c := upcase(sTmp[i]);
      case ord(c) of
        48..57 : sWord := sWord + ByteToBits(ord(c) - 48, 4);
        65..70 : sWord := sWord + ByteToBits(ord(c) - 55, 4);
      end;
    end;
    result := BitsToWord(sWord);
  except
    result := 0;
  end;
end;

function HexToLongWord(s: string): LongWord;
var
  c: char;
  slWord: string;
  i: integer;
  sTmp: string;
begin
  try
    sTmp := s;

    if copy(sTmp, 1, 1) = '$' then
      delete(sTmp, 1, 1);

    if length(sTmp) <> 8 then
      sTmp := StrFixLength(sTmp, 'Left', '0', 8);

    slWord := '';
    for i := 1 to 8 do begin
      c := upcase(sTmp[i]);
      case ord(c) of
        48..57 : slWord := slWord + ByteToBits(ord(c) - 48, 4);
        65..70 : slWord := slWord + ByteToBits(ord(c) - 55, 4);
      end;
    end;
    result := BitsToLongWord(slWord);
  except
    result := 0;
  end;
end;

function  ArrayBoolToInteger(Ar: array of boolean): integer;
var
  i: integer;
begin
  // assumed array contain 31 (not 32) variables
  // bit 31 not used (used to store sign of integer value)
  result := 0;
  try
    for i := 0 to 30 do
      if Ar[i] then
        result := result or (1 shl i);
  except
    result := 0;
  end;
end;

function  IntegerToArrayBool(iVal: integer; var Ar: array of boolean): boolean;
var
  i: integer;
begin
  // assumed array contain 31 (not 32) variables
  // bit 31 not used (used to store sign of integer value)
  try
    for i := 0 to 30 do
      Ar[i] := longBool(iVal and (1 shl i));
    result := true;
  except
    result := false;
  end;
end;


end.
