unit rmPorts;

interface

uses
  Windows;

// common
procedure SetActivePortBaseAddr(LPTAddr, COMAddr, JOYAddr: word);

// joystick
function  JoystickPort_ReadAllValues(
            BaseAddr: word; 
            var A1, A2, A3, A4: integer; 
            var D1 ,D2, D3, D4: boolean; 
            TimeOut: integer): boolean;
function  JoystickPort_ReadAnalogValues(
            BaseAddr: word; 
            var A1, A2, A3, A4: integer; 
            TimeOut: integer): boolean;
function  JoystickPort_ReadDigitalValues(
            BaseAddr: word;
            var D1 ,D2, D3, D4: boolean;
            TimeOut: integer): boolean;

// parallel port
function  ParallelPort_GetMode(
            BaseAddr: word;
            Data: byte): boolean;
function  ParallelPort_SetMode(
            BaseAddr: word;
            iMode: byte): boolean;
function  ParallelPort_WriteAllValues(
            BaseAddr: word;
            bSetBitControl: boolean;
            O_D0, O_D1, O_D2, O_D3, O_D4, O_D5, O_D6, O_D7,
            Strobe, Autofeed, Init, Select: boolean): boolean;
function  ParallelPort_WriteDataValues(
            BaseAddr: word;
            O_D0, O_D1, O_D2, O_D3, O_D4, O_D5, O_D6, O_D7: boolean): boolean;
function  ParallelPort_WriteControlValues(
            BaseAddr: word;
            bDirection: boolean;
            O_Strobe, O_Autofeed, O_Init, O_Select: boolean): boolean;
function  ParallelPort_ReadAllValues(
            BaseAddr: word;
            var I_Busy, I_Ack, I_PaperEnd, I_Select, I_Error: boolean;
            //var I_Strobe, I_AutoFeed, I_Init, I_SelectIn: boolean;
            var I_D0, I_D1, I_D2, I_D3, I_D4, I_D5, I_D6, I_D7: boolean): boolean;
function  ParallelPort_ReadStatusValues(
            BaseAddr: word;
            var I_Busy, I_Ack, I_PaperEnd, I_Select, I_Error: boolean): boolean;
{
function  ParallelPort_ReadCommandValues(
            BaseAddr: word;
            var I_Strobe, I_AutoFeed, I_Init, I_SelectIn: boolean): boolean;
}

function  ParallelPort_ReadDataValues(
            BaseAddr: word;
            var I_D0, I_D1, I_D2, I_D3, I_D4, I_D5, I_D6, I_D7: boolean): boolean;

// serial port
procedure COM_Open;
procedure COM_Close;
function  SerialPort_SetTimerInterruptRequest(state: boolean): boolean;
function  SerialPort_ReadAllValues(
            BaseAddr: word; 
            var I_CTS, I_DSR, I_RI, I_DCD: boolean): boolean;
function  SerialPort_ReadModemStatusValues(
            BaseAddr: word; 
            var I_CTS, I_DSR, I_RI, I_DCD: boolean): boolean;
function  SerialPort_WriteAllValues(
            BaseAddr: word; 
            O_DTR, O_RTS, 
            O_TX: boolean): boolean;
function  SerialPort_WriteModemCommandsValues(
            BaseAddr: word; 
            O_DTR, O_RTS: boolean): boolean;
function  SerialPort_WriteLineCommandsValues(
            BaseAddr: word; 
            O_TX: boolean): boolean;

const

  // Different books says different things about LPT Base adresses
  // Generally we can read this...
  //   LPT1PortBaseAddr = $378;     // used here
  //   LPT2PortBaseAddr = $278;     // used here
  // sometimes we can read this...
  //   LPT1PortBaseAddr = $3BC;
  //   LPT2PortBaseAddr = $378;
  //   LPT3PortBaseAddr = $278;
  // but it seems $378 is generally used than $3BC for LPT1

  // LPT1PortBaseAddr = $3BC;     // 956d
  LPT1PortBaseAddr = $378;        // 888d
  LPT2PortBaseAddr = $278;        // 632d
  COM1PortBaseAddr = $3F8;        // 1016d
  COM2PortBaseAddr = $2F8;        // 760d
  COM3PortBaseAddr = $3E8;        // 1000d
  COM4PortBaseAddr = $2E8;        // 744d
  JoystickPortBaseAddr = $201;

  // type of data to read or write to port
  DataType_Byte = 0;
  DataType_Word = 1;
  DataType_DWord = 2;

var
  FHandle: integer; // here, used to handle com port
  DataType: byte = DataType_DWord;
  KeepOriginalState: boolean;
  RunOnNTPlateform: boolean;
  LPTActivePortBaseAddr: word;
  COMActivePortBaseAddr: word;
  JOYActivePortBaseAddr: word;

  // zlio driver loaded if NT
  LPT_zlIOStarted: boolean;

implementation

uses
  SysUtils,

  // zlPortIO only used for NT/2000 port access
  // NT driver loading made in zlPortIO initialization section
  ZLPortIO;


{
************************************************************************
  Common
************************************************************************
}


procedure SetActivePortBaseAddr(LPTAddr, COMAddr, JOYAddr: word);
begin
  // assign new address
  LPTActivePortBaseAddr := LPTAddr;
  COMActivePortBaseAddr := COMAddr;
  JOYActivePortBaseAddr := JOYAddr;

  // reinit com port
  if COMAddr > - 1 then
  begin
    COM_Close;
    COM_Open;
  end;

end;


{
************************************************************************
  Joystick port
************************************************************************
}

function JoystickPort_ReadAllValues(
           BaseAddr: word; 
           var A1, A2, A3, A4: integer; 
           var D1 ,D2, D3, D4: boolean; 
           TimeOut: integer): boolean;
var
  InputValue: byte;
  OutputValue: smallint;
  A1Stopped, A2Stopped, A3Stopped, A4Stopped: boolean;
  t: longint;
begin
  A1 := 0;
  A2 := 0;
  A3 := 0;
  A4 := 0;
  InputValue := 0;
  OutputValue := 0;
  t := getTickCount;

  // reset counter on joystick port
  try
    asm
      mov dx, BaseAddr
      mov ax, OutputValue
      out dx, al
    end;
  except
    result := false;
    exit;
  end;

  // read joystick port, in loop for analog values,
  try
    repeat
      asm
        mov dx, BaseAddr
        in al, dx
        mov InputValue, al
      end;
      A1Stopped := not ((InputValue and 1) = 1);
      A2Stopped := not ((InputValue and 2) = 2);
      A3Stopped := not ((InputValue and 4) = 4);
      A4Stopped := not ((InputValue and 8) = 8);
      if not A1Stopped then inc(A1);
      if not A2Stopped then inc(A2);
      if not A3Stopped then inc(A3);
      if not A4Stopped then inc(A4);
    until (A1Stopped and A2Stopped and A3Stopped and A4Stopped)
       or ((getTickCount - t) > longint(timeout))
  except
    result := false;
    exit;
  end;

  // read joystick port, direct for digital values
  D1 := not ((InputValue and 16) = 16);
  D2 := not ((InputValue and 32) = 32);
  D3 := not ((InputValue and 64) = 64);
  D4 := not ((InputValue and 128) = 128);

  result := true;
end;

function JoystickPort_ReadAnalogValues(
           BaseAddr: word; 
           var A1, A2, A3, A4: integer; 
           TimeOut: integer): boolean;
var
  InputValue: byte;
  OutputValue: smallint;
  A1Stopped, A2Stopped, A3Stopped, A4Stopped: boolean;
  t: longint;
begin
  A1 := 0;
  A2 := 0;
  A3 := 0;
  A4 := 0;
  InputValue := 0;
  OutputValue := 0;
  t := getTickCount;

  // reset counter on joystick port
  try
    asm
      mov dx, BaseAddr
      mov ax, OutputValue
      out dx, al
    end;
  except
    result := false;
    exit;
  end;

  // read joystick port, in loop for analog values,
  try
    repeat
      asm
        mov dx, BaseAddr
        in al, dx
        mov InputValue, al
      end;
      A1Stopped := not ((InputValue and 1) = 1);
      A2Stopped := not ((InputValue and 2) = 2);
      A3Stopped := not ((InputValue and 4) = 4);
      A4Stopped := not ((InputValue and 8) = 8);
      if not A1Stopped then inc(A1);
      if not A2Stopped then inc(A2);
      if not A3Stopped then inc(A3);
      if not A4Stopped then inc(A4);
    until (A1Stopped and A2Stopped and A3Stopped and A4Stopped)
       or ((getTickCount - t) > longint(timeout))
  except
    result := false;
    exit;
  end;

  result := true;
end;

function JoystickPort_ReadDigitalValues(
           BaseAddr: word; 
           var D1 ,D2, D3, D4: boolean; 
           TimeOut: integer): boolean;
var
  InputValue: byte;
  OutputValue: smallint;
  A1Stopped, A2Stopped, A3Stopped, A4Stopped: boolean;
  t: longint;
begin
  InputValue := 0;

  // read joystick port, direct for digital values
  try
    asm
      mov dx, BaseAddr
      in al, dx
      mov InputValue, al
    end;
    D1 := not ((InputValue and 16) = 16);
    D2 := not ((InputValue and 32) = 32);
    D3 := not ((InputValue and 64) = 64);
    D4 := not ((InputValue and 128) = 128);
  except
    result := false;
    exit;
  end;

  result := true;
end;

{
************************************************************************
  Parallel port
************************************************************************
}

function ParallelPort_GetMode(
           BaseAddr: word;
           Data: byte): boolean;
var
  Addr: DWord;
begin
  // work at Extended Control register address
  Addr := BaseAddr + $0402;

  // get mode
  try
    if ZLIOStarted then
      Data := ZLIOPortRead(Addr, DataType_Byte)
    else
      result := false;
  except
    result := false;
    exit;
  end;
  result := true;

end;

function ParallelPort_SetMode(
           BaseAddr: word;
           iMode: byte): boolean;
var
  Addr: DWord;
  Data: byte;
begin
  // work at Extended Control register address
  Addr := BaseAddr + $0402;

  // set data for mode := 001
  Data := iMode;

  // set mode to iMode value
  try
    if ZLIOStarted then
      ZLIOPortWrite(Addr, DataType_Byte, Data)
    else
      result := false;
  except
    result := false;
    exit;
  end;
  result := true;

end;

function ParallelPort_WriteAllValues(
           BaseAddr: word;
           bSetBitControl: boolean;
           O_D0, O_D1, O_D2, O_D3, O_D4, O_D5, O_D6, O_D7,
           Strobe, Autofeed, Init, Select: boolean): boolean;
var
  WriteDataOK, WriteControlOK: boolean;
begin
  WriteDataOK :=
    ParallelPort_WriteDataValues(
      BaseAddr,
      O_D0, O_D1, O_D2, O_D3, O_D4, O_D5, O_D6, O_D7);
  WriteControlOK :=
    ParallelPort_WriteControlValues(
      BaseAddr,
      bSetBitControl,
      Strobe, Autofeed, Init, Select);
  result := WriteDataOK and WriteControlOK;
end;

function ParallelPort_WriteDataValues(BaseAddr: word; O_D0, O_D1, O_D2, O_D3, O_D4, O_D5, O_D6, O_D7: boolean): boolean;
var
  Data: DWord;
  Addr: DWord;
  i: byte;
begin
  // work at Data register address
  Addr := BaseAddr + 0;

  // Get data value
  Data := 0;
  if O_D0 then Data := Data + 1;
  if O_D1 then Data := Data + 2;
  if O_D2 then Data := Data + 4;
  if O_D3 then Data := Data + 8;
  if O_D4 then Data := Data + 16;
  if O_D5 then Data := Data + 32;
  if O_D6 then Data := Data + 64;
  if O_D7 then Data := Data + 128;

  // write value on LPT port if system IS NOT WinNT
  if Win32Platform <> VER_PLATFORM_WIN32_NT then
  begin
    try
      Case DataType of
        DataType_Byte :
          asm
            mov edx, Addr
            mov eax, Data
            out dx, al
          end;
        DataType_Word :
          asm
            mov edx, Addr
            mov eax, Data
            out dx, ax
          end;
        DataType_DWord:
          asm
            mov edx, Addr
            mov eax, Data
            out dx, eax
          end;
       end;
      {
      asm
        mov dx, Addr
        mov ax, Data
        out dx, al
      end;
      }
    except
      result := false;
      exit;
    end;

  end
  else

  // write value on LPT port if system IS WinNT
  begin
    try
      if ZLIOStarted then
        ZLIOPortWrite(Addr, DataType, Data)
      else
        result := false;
    except
      result := false;
      exit;
    end;

  end;  // if then else

  result := true;
end;

function ParallelPort_WriteControlValues(
           BaseAddr: word;
           bDirection: boolean;
           O_Strobe, O_Autofeed, O_Init, O_Select: boolean): boolean;
var
  Data: DWord;
  Addr: DWord;
begin
  // work at Command register address
  Addr := BaseAddr + 2;

  // Get data value
  Data := 0;
  if not O_Strobe then Data := Data + 1;
  if not O_Autofeed then Data := Data + 2;
  if O_Init then Data := Data + 4;
  if not O_Select then Data := Data + 8;

  // set direction
  if bDirection then Data := Data + 32;

  // write value on port LPT if system IS NOT WinNT
  if Win32Platform <> VER_PLATFORM_WIN32_NT then begin
    try
      Case DataType of
        DataType_Byte :
          asm
            mov edx, Addr
            mov eax, Data
            out dx, al
          end;
        DataType_Word :
          asm
            mov edx, Addr
            mov eax, Data
            out dx, ax
          end;
        DataType_DWord:
          asm
            mov edx, Addr
            mov eax, Data
            out dx, eax
          end;
       end;
      {
      asm
        mov dx, Addr
        mov ax, Data
        out dx, al
      end;
      }

    except
      result := false;
      exit;
    end;
  end
  else

  // write value on LPT port if system IS WinNT
  begin
    try
      if ZLIOStarted then
        ZLIOPortWrite(Addr, DataType, Data)
      else
        result := false;
    except
      result := false;
      exit;
    end;

  end;  // if then else

  result := true;
end;

function  ParallelPort_ReadAllValues(
            BaseAddr: word;
            var I_Busy, I_Ack, I_PaperEnd, I_Select, I_Error: boolean;
            //var I_Strobe, I_AutoFeed, I_Init, I_SelectIn: boolean;
            var I_D0, I_D1, I_D2, I_D3, I_D4, I_D5, I_D6, I_D7: boolean): boolean;
var
  ReadDataOK, ReadStatusOK, ReadCommandOK: boolean;
begin
  // read Status lines
  ReadStatusOK :=
    ParallelPort_ReadStatusValues(
      BaseAddr,
      I_Busy, I_Ack, I_PaperEnd, I_Select, I_Error);

  // set to reverse mode (for read data pin)
  ParallelPort_WriteControlValues(
    BaseAddr,
    true,   // direction bit set to reverse mode
    false, false, false, false);

  // read Data lines
  ReadDataOK :=
    ParallelPort_ReadDataValues(
      BaseAddr,
      I_D0, I_D1, I_D2, I_D3, I_D4, I_D5, I_D6, I_D7);

  // set to forward mode (for read Command lines and Status lines)
  {
  ParallelPort_WriteControlValues(
    BaseAddr,
    false,   // direction bit set to forward mode
    false, false, false, false);
  }

  // read Command lines
  {
  ReadCommandOK :=
    ParallelPort_ReadCommandValues(
      BaseAddr,
      I_Strobe, I_AutoFeed, I_Init, I_SelectIn);
  }

  // read OK ?
  result := ReadStatusOK and ReadDataOK and ReadCommandOK;
end;

function  ParallelPort_ReadStatusValues(
            BaseAddr: word; 
            var I_Busy, I_Ack, I_PaperEnd, I_Select, I_Error: boolean): boolean;
var
  Data: DWord;
  Addr: DWord;
begin
  // work at Status register address
  Addr := BaseAddr + 1;

  // init status (only used if error occurs during read)
  I_Busy := false;
  I_Ack := false;
  I_PaperEnd := false;
  I_Select := false;
  I_Error := false;

  // read status on LPT port if system IS NOT WinNT
  if Win32Platform <> VER_PLATFORM_WIN32_NT then
  begin
    try
      Case DataType of
        DataType_Byte :
          asm
            mov edx,Addr
            xor eax,eax
            in al,dx
            mov Data,eax
          end;
        DataType_Word :
          asm
            mov edx,Addr
            xor eax,eax
            in ax,dx
            mov Data,eax
          end;
        DataType_DWord:
          asm
            mov edx,Addr
            xor eax,eax
            in eax,dx
            mov Data,eax
          end;
       end;
      {
      asm
        mov dx, Addr
        in al, dx
        mov Data, al
      end;
      }
    except
      result := false;
      exit;
    end;
  end
  else

  // read status value on LPT port if system IS WinNT
  begin
    try
      if ZLIOStarted then
        Data := ZLIOPortRead(Addr, DataType)
      else
      begin
        Data := 0;
        result := false;
      end;
    except
      result := false;
      exit;
    end;

  end;

  // extract bits value for individual status line
  I_Busy := (((Data and 128) div 128) = 1);
  I_Ack := (((Data and 64) div 64) = 1);
  I_PaperEnd := (((Data and 32) div 32) = 1);
  I_Select := (((Data and 16) div 16) = 1);
  I_Error := (((Data and 8) div 8) = 1);

  // invert state ?
  KeepOriginalState := false;      
  if not KeepOriginalState then
  begin
    I_Busy := not I_Busy;
    // I_Ack := I_Ack;
    // I_PaperEnd := I_PaperEnd;
    // I_Select := I_Select;
    // I_Error := I_Error;
  end;

  // debug
  {
  I_Busy := false;
  I_Ack := false;
  I_PaperEnd := false;
  I_Select := false;
  I_Error := true;
  }

  result := true;
end;

{
function  ParallelPort_ReadCommandValues(
            BaseAddr: word;
            var I_Strobe, I_AutoFeed, I_Init, I_SelectIn: boolean): boolean;
var
  Data: DWord;
  Addr: DWord;
begin
  // work at Command register address
  Addr := BaseAddr + 2;

  // init commands (only used if error occurs during read)
  I_SelectIn := false;
  I_Init := false;
  I_AutoFeed := false;
  I_Strobe := false;

  // read commands lines on LPT port if system IS NOT WinNT
  if Win32Platform <> VER_PLATFORM_WIN32_NT then begin
    try
      Case DataType of
        DataType_Byte :
          asm
            mov edx,Addr
            xor eax,eax
            in al,dx
            mov Data,eax
          end;
        DataType_Word :
          asm
            mov edx,Addr
            xor eax,eax
            in ax,dx
            mov Data,eax
          end;
        DataType_DWord:
          asm
            mov edx,Addr
            xor eax,eax
            in eax,dx
            mov Data,eax
          end;
       end;
      {
      asm
        mov dx, Addr
        in al, dx
        mov Data, al
      end;
      }
      {
    except
      result := false;
      exit;
    end;
  end
  else

  // read commands lines on LPT port if system IS WinNT
  begin
    try
      if ZLIOStarted then
        Data := ZLIOPortRead(Addr, DataType_Byte)
      else
      begin
        Data := 0;
        result := false;
      end;
    except
      result := false;
      exit;
    end;

  end;

  // extract bits value for individual status line
  I_SelectIn := (((Data and 8) div 8) = 1);
  I_Init := (((Data and 4) div 4) = 1);
  I_AutoFeed := (((Data and 2) div 2) = 1);
  I_Strobe := (((Data and 1) div 1) = 1);

  // invert state ?
  KeepOriginalState := false;
  if not KeepOriginalState then
  begin
    I_Strobe := not I_Strobe;
    I_AutoFeed := not I_AutoFeed;
    // I_Init := I_Init;
    I_SelectIn := not I_SelectIn;
  end;

  // debug
  {
  I_SelectIn := false;
  I_Init := false;
  I_AutoFeed := true;
  I_Strobe := false;

  result := true;
end;
}

function  ParallelPort_ReadDataValues(
            BaseAddr: word; 
            var I_D0, I_D1, I_D2, I_D3, I_D4, I_D5, I_D6, I_D7: boolean): boolean;
var
  Data: DWord;
  Addr: DWord;
begin
  //Set bit 5 to "1" to allow reading I/O (Data) ports
  ParallelPort_WriteControlValues
    (BaseAddr, true, false, false, false, false);

  // work at Data register address
  Addr := BaseAddr + 0;

  // init data (only used if error occurs during read)
  I_D0 := false;
  I_D1 := false;
  I_D2 := false;
  I_D3 := false;
  I_D4 := false;
  I_D5 := false;
  I_D6 := false;
  I_D7 := false;

  // read data value on LPT port if system IS NOT WinNT
  if Win32Platform <> VER_PLATFORM_WIN32_NT then
  begin
    try
      Case DataType of
        DataType_Byte :
          asm
            mov edx,Addr
            xor eax,eax
            in al,dx
            mov Data,eax
          end;
        DataType_Word :
          asm
            mov edx,Addr
            xor eax,eax
            in ax,dx
            mov Data,eax
          end;
        DataType_DWord:
          asm
            mov edx,Addr
            xor eax,eax
            in eax,dx
            mov Data,eax
          end;
       end;
      {
      asm
        mov dx, Addr
        in al, dx
        mov Data, al
      end;
      }
    except
      result := false;
      exit;
    end;
  end
  else

  // read data value on LPT port if system IS WinNT
  begin
    try
      if ZLIOStarted then
        Data := ZLIOPortRead(Addr, DataType_Byte)
      else
      begin
        Data := 0;
        result := false;
      end;
    except
      result := false;
      exit;
    end;

  end;

  // extract bits value for individual data line
  I_D7 := (((Data and 128) div 128) = 1);
  I_D6 := (((Data and 64) div 64) = 1);
  I_D5 := (((Data and 32) div 32) = 1);
  I_D4 := (((Data and 16) div 16) = 1);
  I_D3 := (((Data and 8) div 8) = 1);
  I_D2 := (((Data and 4) div 4) = 1);
  I_D1 := (((Data and 2) div 2) = 1);
  I_D0 := (((Data and 1) div 1) = 1);

  // invert state ?
  //  no !!!

  // debug
  {
  I_D0 := false;
  I_D1 := false;
  I_D2 := false;
  I_D3 := false;
  I_D4 := true;
  I_D5 := false;
  I_D6 := false;
  I_D7 := false;
  }

  //Set bit 5 to "0" to allow writting I/O (Data) ports
  ParallelPort_WriteControlValues
    (BaseAddr, false, false, false, false, false);


  result := true;
end;


{
************************************************************************
  serial port
************************************************************************
}

procedure COM_Open;
var
  sPort: String;
begin
  FHandle := INVALID_HANDLE_VALUE;
  case COMActivePortBaseAddr of
    COM1PortBaseAddr: sPort := 'COM1';
    COM2PortBaseAddr: sPort := 'COM2';
    COM3PortBaseAddr: sPort := 'COM3';
    COM4PortBaseAddr: sPort := 'COM4';
  end;
  FHandle := CreateFile(
    PChar(sPort),
    GENERIC_READ or GENERIC_WRITE,
    0,
    nil,
    OPEN_EXISTING,
    FILE_FLAG_OVERLAPPED,
    0);
end;

procedure COM_Close;
begin
  if FHandle <> INVALID_HANDLE_VALUE then
    CloseHandle(FHandle);
end;

function SerialPort_SetTimerInterruptRequest(state: boolean): boolean;
begin
  // interrupt request autorized
  if state then begin
    try
      asm
        STI
      end;
    except
      result := false;
    end;

  // interrupt request not autorized
  end else begin
    try
      asm
        CLI
      end;
    except
      result := false;
    end;
  end;

  result := true;
end;

function SerialPort_ReadAllValues(
           BaseAddr: word; 
           var I_CTS, I_DSR, I_RI, I_DCD: boolean): boolean;
begin
  result := SerialPort_ReadModemStatusValues(
              BaseAddr, 
              I_CTS, I_DSR, I_RI, I_DCD);
end;

function SerialPort_ReadModemStatusValues(
           BaseAddr: word; 
           var I_CTS, I_DSR, I_RI, I_DCD: boolean): boolean;
var
  Data: DWord;
  Addr: DWord;
begin
  if FHandle = INVALID_HANDLE_VALUE then
    exit;

  Addr := BaseAddr + 6;

  // read value on COM port if system IS NOT WinNT
  if Win32Platform <> VER_PLATFORM_WIN32_NT then
  begin
    try
      Case DataType of
        DataType_Byte :
          asm
            mov edx,Addr
            xor eax,eax
            in al,dx
            mov Data,eax
          end;
        DataType_Word :
          asm
            mov edx,Addr
            xor eax,eax
            in ax,dx
            mov Data,eax
          end;
        DataType_DWord:
          asm
            mov edx,Addr
            xor eax,eax
            in eax,dx
            mov Data,eax
          end;
       end;
      {
      asm
        mov dx, Addr
        in al, dx
        mov Data, al
      end;
      }
    except
      result := false;
      exit;
    end;
  end
  else

  // read status value on COM port if system IS WinNT
  begin
    try
      if ZLIOStarted then
        Data := ZLIOPortRead(Addr, DataType)
      else
      begin
        Data := 0;
        result := false;
      end;
    except
      result := false;
      exit;
    end;

  end;

  // extract bits value for individual status line
  I_DCD := ((Data and 128) = 128);
  I_RI := ((Data and 64) = 64);
  I_DSR := ((Data and 32) = 32);
  I_CTS := ((Data and 16) = 16);

  result := true;
end;

function SerialPort_WriteAllValues(
           BaseAddr: word; 
           O_DTR, O_RTS, O_TX: boolean): boolean;
var
  WriteModemOK, WriteLineOK: boolean;
begin
  WriteModemOK := 
    SerialPort_WriteModemCommandsValues(
      BaseAddr, 
      O_DTR, O_RTS);
  WriteLineOK := 
    SerialPort_WriteLineCommandsValues(
      BaseAddr, 
      O_TX);
  result := WriteModemOK and WriteLineOK;
end;

function SerialPort_WriteModemCommandsValues(
           BaseAddr: word; 
           O_DTR, O_RTS: boolean): boolean;
var
  Data: DWord;
  Addr: DWord;
begin
  if FHandle = INVALID_HANDLE_VALUE then
    exit;

  Data := 0;
  Addr := BaseAddr + 4;
  if O_DTR then Data := Data + 1;    // DTR = bit 0
  if O_RTS then Data := Data + 2;    // RTS = bit 1

  // write value on COM port if system IS NOT WinNT
  if Win32Platform <> VER_PLATFORM_WIN32_NT then
  begin

    try
      Case DataType of
        DataType_Byte :
          asm
            mov edx, Addr
            mov eax, Data
            out dx, al
          end;
        DataType_Word :
          asm
            mov edx, Addr
            mov eax, Data
            out dx, ax
          end;
        DataType_DWord:
          asm
            mov edx, Addr
            mov eax, Data
            out dx, eax
          end;
       end;
      {
      asm
        mov dx, Addr
        mov ax, Data
        out dx, al
      end;
      }

    except
      result := false;
      exit;
    end;

  end
  else

  // write value on COM port if system IS WinNT
  begin
    try
      if ZLIOStarted then
        ZLIOPortWrite(Addr, DataType, Data)
      else
        result := false;
    except
      result := false;
      exit;
    end;

  end;  // if then else

  result := true;
end;

function SerialPort_WriteLineCommandsValues(
           BaseAddr: word; 
           O_TX: boolean): boolean;
var
  Data: DWord;
  Addr: DWord;
begin
  if FHandle = INVALID_HANDLE_VALUE then
    exit;

  Data := 0;
  Addr := BaseAddr + 3;
  if O_TX then Data := Data + 64;    // TX = bit 6

  // write value on COM port if system IS NOT WinNT
  if Win32Platform <> VER_PLATFORM_WIN32_NT then
  begin
    try
      Case DataType of
        DataType_Byte :
          asm
            mov edx, Addr
            mov eax, Data
            out dx, al
          end;
        DataType_Word :
          asm
            mov edx, Addr
            mov eax, Data
            out dx, ax
          end;
        DataType_DWord:
          asm
            mov edx, Addr
            mov eax, Data
            out dx, eax
          end;
       end;
      {
      asm
        mov dx, Addr
        mov ax, Data
        out dx, al
      end;
      }

    except
      result := false;
      exit;
    end;

  end
  else

  // write value on COM port if system IS WinNT
  begin
    try
      if ZLIOStarted then
        ZLIOPortWrite(Addr, DataType, Data)
      else
        result := false;
    except
      result := false;
      exit;
    end;

  end;  // if then else

  result := true;
end;


initialization
  // ZLPortIO.zliostart; // made in initialization section of zlPortIO unit
  LPT_zlIOStarted := zlIOStarted;

finalization
  COM_Close;
  // ZLPortIO.zliostop; // made in initialization section of zlPortIO unit


end.
